function [model,stats,p,FittedParms,FitVal, EXITFLAG] = FitParmsExactID(p_base,moment_specs, useDevice, DisplayInterval,...
    T,TBurn,Us_z,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry)
% use same shocks to get rid of simulation noise

    bDEBUG_DISPLAY = true;
    
    % DiffMinChange: Minimum change in variables for finite-difference gradients (a positive scalar). The default is 0.
    % TypicalX
    
    fsolve_opts = optimset('display','iter-detailed','TypicalX',ones(moment_specs.NumParms,1),'DiffMinChange',0.025,'TolFun',1e-2);
    
    X0 = zeros(moment_specs.NumParms,1);
    for i = 1:moment_specs.NumParms
        X0(i) = feval(moment_specs.parm_transform_inv{i},moment_specs.parm_init_guess(i));
    end

    [XFit, FitVal, EXITFLAG] = fsolve(@(X)Moments(X),X0,fsolve_opts);
    
    FittedParms = zeros(moment_specs.NumParms,1);
    for i = 1:moment_specs.NumParms
        FittedParms(i) = feval(moment_specs.parm_transform{i},XFit(i));
    end

    function Z = Moments(X)
        
        Z = zeros(moment_specs.NumParms,1);
        p = p_base;
        
        % assign parameters
        for idx = 1:moment_specs.NumParms
            temp_parm_val = feval(moment_specs.parm_transform{idx},X(idx));
            p = setfield(p,moment_specs.parm_names{idx},temp_parm_val);
            if bDEBUG_DISPLAY
                fprintf('%s set to %g.\n', moment_specs.parm_names{idx}, temp_parm_val);
            end
        end
        
        % solve model on GPU
        model_mex = mexSolveModelGivenParms(p, useDevice, DisplayInterval);
        
        % convert from c++ output to matlab output
        model = ConvertCPPtoMAT(p,model_mex);
        
        % simulate
        [~,stats] = SimSinglePath_mex_output(p,model,T,TBurn,...
            Us_z,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry);
        
        % output
        for idx = 1:moment_specs.NumParms
            target_stat = getfield(stats,moment_specs.target_stat_names{idx});
            temp_target = moment_specs.target_stat_vals(idx);
            Z(idx) = (target_stat - temp_target)/temp_target;
            if bDEBUG_DISPLAY
                fprintf('%s has rel. err. %g (percent).\n', moment_specs.target_stat_names{idx}, 100*Z(idx));
            end
        end
        
    end

end