function [model,stats,p,FittedParms,FitVal, EXITFLAG] = FitParmsExactID_Parallel_3moments(p_base, pool, target_debt_gdp, target_mean_spread, target_std_spread, ...
    beta_guess, frac_loss_y1_guess, slope_guess, ...
    DisplayInterval,T,TBurn,Us_z,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry)
% use same shocks to get rid of simulation noise
% moments are fixed: debt level, mean spreads, vol of spreads.
% parameters: beta, level of costs, slope of costs

    if pool.NumWorkers>gpuDeviceCount
        error('Insufficient number of GPU devices');
    end
    
    % DiffMinChange: Minimum change in variables for finite-difference gradients (a positive scalar). The default is 0.
    % TypicalX
    
    TypX = [4.6;0.07;0.3];
    fsolve_opts = optimset('display','iter-detailed','TypicalX',TypX,'DiffMinChange',0.01,'TolFun',1e-2,'UseParallel',true);
    
    X0 = zeros(3,1);
    X0(1) = -log(1/beta_guess - 1);
    X0(2) = frac_loss_y1_guess;
    X0(3) = slope_guess;

    [XFit, FitVal, EXITFLAG] = fsolve(@(X)Moments(X),X0,fsolve_opts);
    
    FittedParms = zeros(3,1);
    FittedParms.beta = 1/(1 + exp(-XFit(1)));
    FittedParms.d_y = XFit(2) - XFit(3);
    FittedParms.ds_yy = XFit(3);

    function Z = Moments(X)
        
        t = getCurrentTask();
        if isempty(t)
            useDevice = 0;
        else
            useDevice = t.ID - 1;
        end
        
        Z = zeros(3,1);
        p = p_base;
        
        % assign parameters
        p.beta = 1/(1 + exp(-X(1)));
        p.d_y = X(2) - X(3);
        p.d_yy = X(3);
        
        % solve model on GPU
        model_mex = mexSolveModelGivenParms(p, useDevice, DisplayInterval);
        
        % convert from c++ output to matlab output
        model = ConvertCPPtoMAT(p,model_mex);
        
        % simulate
        [~,stats] = SimSinglePath_mex_output(p,model,T,TBurn,...
            Us_z,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry);
        
        % output
        Z(1) = 100*(stats.DebtToOutput_mean - target_debt_gdp);
        Z(2) = 1000*(stats.Spread_mean - target_mean_spread);
        Z(3) = 1000*(stats.Spread_std - target_std_spread);
        
    end

end