function mod = ComputeModelAuxiliary(p,mod)
% things to compute:
% 1. term structure of default probabilities
    
    [qND_noliq,qD_noliq] = CalcBondPrices_noliq(p,mod);
    
    mod.qND_noliq = qND_noliq;
    mod.qD_noliq = qD_noliq;

end

function [qND_noliq,qD_noliq] = CalcBondPrices_noliq(p,model)

    qND_noliq = zeros(size(model.qNDH));
    qD_noliq = zeros(size(model.qDH));
    qND_noliq_old = zeros(size(model.qNDH));
    qD_noliq_old = zeros(size(model.qDH));
    qND_noliq_haircut = zeros(size(model.qNDH));
    
    bContinue = true;
    iter = 0;
    
    default_threshold_eps = model.defthresh;
    debt_policy = model.policy;
    
    rec_frac = min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b);
    rec_frac = permute(repmat(reshape(rec_frac,[],1),[1 p.gridsize_tfp]),[2 1]);
    
    while bContinue
        
        % integrate over noise
        parfor idx_y = 1:p.gridsize_tfp
            qND_noliq(idx_y,:) = CalcBondPrices_noliq_given_y(p,default_threshold_eps(idx_y,:),debt_policy(idx_y,:),...
                qND_noliq_old(idx_y,:),qD_noliq_old(idx_y,:));
        end
        
        % z'|z
        qND_noliq = p.prob_y*qND_noliq/(1+p.rH);
        qD_noliq1 = p.prob_y*qD_noliq_old;
        
        for idx_y = 1:p.gridsize_tfp
            qND_noliq_haircut(idx_y,:) = interp1(p.grid_b,qND_noliq(idx_y,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest');
        end

        qD_noliq = max(0,(1-p.reenterprob)*qD_noliq1/(1+p.rH) ...
            + p.reenterprob*rec_frac.*qND_noliq_haircut);
        
        iter = iter + 1;
        
        err_qND = max(abs(reshape(qND_noliq-qND_noliq_old,[],1)));
        err_qD = max(abs(reshape(qD_noliq-qD_noliq_old,[],1)));
        
        if iter>p.maxiters || max([err_qND err_qD])<p.errtolQ
            bContinue = false;
        else
            qND_noliq_old = qND_noliq;
            qD_noliq_old = qD_noliq;
        end
        
        fprintf('iter %g, err(qND)=%g, err(qD)=%g.\n', iter, err_qND, err_qD);
        
    end

end

function q = CalcBondPrices_noliq_given_y(p,default_threshold_eps,debt_policy,qND,qD)

    q = zeros(1,p.gridsize_b);
    
    for idx_b = 1:p.gridsize_b
        q(idx_b) = CalcBondPrices_noliq_given_y_b(p,default_threshold_eps(idx_b),debt_policy{idx_b},qND,qD(idx_b));
    end

end

function q = CalcBondPrices_noliq_given_y_b(p,default_threshold_eps,debt_policy,qND,qD)

    q = 0; % \int (1-D(y,e,b))*qCL(y,b'(y,e,b)) de, with (y,b) being parameters
    
    if ~debt_policy.EmptyChoiceSet
        for j = 1:debt_policy.NumRegions
            region_lb = debt_policy.eps_mesh(j);
            region_ub = debt_policy.eps_mesh(j+1);
            % update ceC and qND_H and qND_L
            if default_threshold_eps>=region_ub
                % default in the entire region
                mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                    normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                q = q + qD*mass;
            else
                idx_bnext = debt_policy.idx_bnext(j);
                if default_threshold_eps<=region_lb
                    % no default in the entire region
                    % f(Clb,RRA,BETA,C,W)
                    mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    q = q + (p.debt_m+(1-p.debt_m)*p.debt_z...
                        +(1-p.debt_m)*qND(idx_bnext))*mass;
                else
                    % default then no default
                    massdef = normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    massnodef = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd);
                    q = q + ...
                        (p.debt_m+(1-p.debt_m)*p.debt_z+(1-p.debt_m)*qND(idx_bnext))*massnodef...
                        +qD*massdef;
                end
            end
        end
        q = max(0,q)/p.ggq_mmass;
    else
        % debt_policy.EmptyChoiceSet = true.
        % so default throughout
        q = max(0,qD);
    end    

end