function ProcessGPUOutputs_liq(gpufoldername,savefilename,precision)
% syntax: ProcessGPUOutputs(gpufoldername,savefilename,NumCores)

    %% load data
    % formats are z,b
    % precision = get_typesize([gpufoldername,'/typesize']);
    
    if isunix
        folder_divisor = '/';
    else
        folder_divisor = '\';
    end
    
    p = read_binary_parms_liq([gpufoldername,folder_divisor,'parms'],precision);
    [~,CVD] = read_2d_binary_file([gpufoldername,folder_divisor,'CVD'],precision);
    [~,CVND] = read_2d_binary_file([gpufoldername,folder_divisor,'CVND'],precision);
    [~,qHD] = read_2d_binary_file([gpufoldername,folder_divisor,'qHD'],precision);
    [~,qLD] = read_2d_binary_file([gpufoldername,folder_divisor,'qLD'],precision);
    [~,qHND] = read_2d_binary_file([gpufoldername,folder_divisor,'qHND'],precision);
    [~,qLND] = read_2d_binary_file([gpufoldername,folder_divisor,'qLND'],precision);
    [~,defprob] = read_2d_binary_file([gpufoldername,folder_divisor,'defprob'],precision);
    [~,defthresh] = read_2d_binary_file([gpufoldername,folder_divisor,'defthresh'],precision);
    
    [~,CVD_old] = read_2d_binary_file([gpufoldername,folder_divisor,'CVD_old'],precision);
    [~,CVND_old] = read_2d_binary_file([gpufoldername,folder_divisor,'CVND_old'],precision);
    [~,qHD_old] = read_2d_binary_file([gpufoldername,folder_divisor,'qHD_old'],precision);
    [~,qLD_old] = read_2d_binary_file([gpufoldername,folder_divisor,'qLD_old'],precision);
    [~,qHND_old] = read_2d_binary_file([gpufoldername,folder_divisor,'qHND_old'],precision);
    [~,qLND_old] = read_2d_binary_file([gpufoldername,folder_divisor,'qLND_old'],precision);
    [~,defprob_old] = read_2d_binary_file([gpufoldername,folder_divisor,'defprob_old'],precision);

    %% set up parfor
    
%     NumPhysicalCoresOnMachine = feature('numCores');
%     if nargin<=2
%         NumCores = min(min(NumPhysicalCoresOnMachine,24),p.gridsize_tfp);
%     end
%     
%     tempdirname = createTempDir;
%     pc = parcluster();
%     pc.JobStorageLocation = tempdirname;
%     pool = parpool(pc,NumCores);

    %% solve model
    
    VD_mat = zeros(p.gridsize_tfp,p.gridsize_b);
    debt_policy_mat = cell(p.gridsize_tfp,p.gridsize_b);
    default_threshold_eps_mat = zeros(p.gridsize_tfp,p.gridsize_b);
    ceND_new_integrate_eps_mat = zeros(p.gridsize_tfp,p.gridsize_b);
    qND_H_new_integrate_eps_mat = zeros(p.gridsize_tfp,p.gridsize_b);
    qND_L_new_integrate_eps_mat = zeros(p.gridsize_tfp,p.gridsize_b);
    defprob_new_integrate_eps = zeros(p.gridsize_tfp,p.gridsize_b);
    qND_diff = zeros(p.gridsize_tfp,p.gridsize_b);
    qD_diff = zeros(p.gridsize_tfp,p.gridsize_b);
    
    parfor idx_y = 1:p.gridsize_tfp
    % for idx_y = 1:p.gridsize_tfp
        [VD_mat(idx_y,:),debt_policy_mat(idx_y,:),default_threshold_eps_mat(idx_y,:),ceND_new_integrate_eps_mat(idx_y,:),...
            qND_H_new_integrate_eps_mat(idx_y,:),qND_L_new_integrate_eps_mat(idx_y,:),defprob_new_integrate_eps(idx_y,:),...
            qND_diff(idx_y,:),qD_diff(idx_y,:)] = ...
            VFI_given_y(p,idx_y,CVND_old(idx_y,:),CVD_old(idx_y,:),...
                qHND_old(idx_y,:),qLND_old(idx_y,:),qHD_old(idx_y,:),qLD_old(idx_y,:),defprob_old(idx_y,:));
    end
    
    [ceD_mat,ceC_mat,qND_H_mat,qND_L_mat,qD_H_mat,qD_L_mat,defprob_mat,ellND,ellD] = UpdateVals(p,VD_mat,ceND_new_integrate_eps_mat,...
        qND_H_new_integrate_eps_mat,qND_L_new_integrate_eps_mat,defprob_new_integrate_eps,qHD_old,qLD_old,qND_diff,qD_diff);
    
    %% clean up
%     delete(pool)
%     rmdir(tempdirname,'s');
    
%     clear pool tempdirname pc
    err_CVD = max(reshape(abs(ceD_mat - CVD),[],1))
    err_CVND = max(reshape(abs(ceC_mat - CVND),[],1))
    err_qNDH = max(reshape(abs(qHND - qND_H_mat),[],1))
    err_qNDL = max(reshape(abs(qLND - qND_L_mat),[],1))
    err_qDH = max(reshape(abs(qHD - qD_H_mat),[],1))
    err_qDL = max(reshape(abs(qLD - qD_L_mat),[],1))
    err_defprob = max(reshape(abs(defprob - defprob_mat),[],1))
    err_defthresh = max(reshape(abs(defthresh - default_threshold_eps_mat),[],1))
    
    
    %% save data
    model.VD = VD_mat;
    model.CVD = ceD_mat;
    model.CVND = ceC_mat;
    model.policy = debt_policy_mat;
    model.defthresh = default_threshold_eps_mat;
    model.defprob = defprob_mat;
    model.qNDH = qND_H_mat;
    model.qNDL = qND_L_mat;
    model.qDH = qD_H_mat;
    model.qDL = qD_L_mat;
    model.ellND = ellND;
    model.ellD = ellD;
    
    model.err_CVD = err_CVD;
    model.err_CVND = err_CVND;
    model.err_qNDH = err_qNDH;
    model.err_qNDL = err_qNDL;
    model.err_qDH = err_qDH;
    model.err_qDL = err_qDL;
    model.err_defprob = err_defprob;
    model.err_defthresh = err_defthresh;
    
%     model.qNDH_old = qHND_old;
%     model.qNDL_old = qLND_old;
%     model.qDH_old = qHD_old;
%     model.qDL_old = qLD_old;
    
    % model = ComputeModelAuxiliary(p,model);
    model = ComputeModelAuxiliaryEfficient(p,model);
    
    save(savefilename,'model','p');

end

function [VD_new,debt_policy,default_threshold_eps,ceND_new_integrate_eps,...
    qND_H_new_integrate_eps,qND_L_new_integrate_eps,defprob_new_integrate_eps,qND_diff,qD_diff] = ...
        VFI_given_y(p,idx_y,ceC,ceD,qND_H,qND_L,qD_H,qD_L,defprob)

    ceC = reshape(ceC,[],1);
    ceD = reshape(ceD,[],1);
    qND_H = reshape(qND_H,[],1);
    qND_L = reshape(qND_L,[],1);
    qD_H = reshape(qD_H,[],1);
    qD_L = reshape(qD_L,[],1);
    defprob = reshape(defprob,[],1);

    VD_new = zeros(p.gridsize_b,1);
    debt_policy = cell(p.gridsize_b,1);
    default_threshold_eps = zeros(p.gridsize_b,1);
    ceND_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_H_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_L_new_integrate_eps = zeros(p.gridsize_b,1);
    defprob_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_diff = zeros(p.gridsize_b,1);
    qD_diff = zeros(p.gridsize_b,1);

    for idx_b = 1:p.gridsize_b
%         if idx_y==10 && idx_b==69
%             fprintf('\n')
%         end
        [VD_new(idx_b),debt_policy{idx_b},default_threshold_eps(idx_b),ceND_new_integrate_eps(idx_b),...
            qND_H_new_integrate_eps(idx_b),qND_L_new_integrate_eps(idx_b),defprob_new_integrate_eps(idx_b),...
                qND_diff(idx_b),qD_diff(idx_b)] = ...
                VFI_given_y_b(p,idx_y,idx_b,ceC,ceD,qND_H,qND_L,qD_H,qD_L,defprob);
    end

end

function [ceDwithhaircut,ceC,qND_H,qND_L,qD_H,qD_L,defprob,ellND,ellD] = UpdateVals(p,VD_new,ceC_new_integrate_eps,...
    qND_H_new_integrate_eps,qND_L_new_integrate_eps,defprob_new_integrate_eps,qD_H_old,qD_L_old,qND_diff,qD_diff)
%   qND_diff = (1-m)*qdiff*NODEF + qdiff*DEF, integrated over smoothing shock
%   qD_diff = qdiff, q=qD

    % update value functions
    ceD1 = p.prob_y*VD_new; % E[VD(y',b)|y]
    ceC = p.prob_y*ceC_new_integrate_eps; %E[max{VD(y',b'),VC(y',e',b')}|y]
    ceC_haircut = zeros(p.gridsize_tfp,p.gridsize_b); % reentering at (1-L)*(1-deltaL)*b
    for idx_y = 1:p.gridsize_tfp
        ceC_haircut(idx_y,:) = interp1(p.grid_b,ceC(idx_y,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest'); % linear is most robust
    end
    ceDwithhaircut = (1-p.reenterprob)*ceD1 + p.reenterprob*ceC_haircut;
    
    % update bond prices, check these for consistency...
    qND_H = p.prob_y*qND_H_new_integrate_eps/(1+p.rH);
    qND_L = p.prob_y*qND_L_new_integrate_eps/(1+p.rL);
    
    qD_H1 = p.prob_y*((1-p.prob_liqshock)*qD_H_old+p.prob_liqshock*qD_L_old);
    qD_L1 = p.prob_y*max(0,(1-p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D))*qD_L_old+p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D)*qD_H_old - p.holding_cost);
    qND_H_haircut = zeros(p.gridsize_tfp,p.gridsize_b);
    qND_L_haircut = zeros(p.gridsize_tfp,p.gridsize_b);
    for idx_y = 1:p.gridsize_tfp
        qND_H_haircut(idx_y,:) = interp1(p.grid_b,qND_H(idx_y,:),min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b).*p.grid_b,'nearest');
        qND_L_haircut(idx_y,:) = interp1(p.grid_b,qND_L(idx_y,:),min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b).*p.grid_b,'nearest');
    end
    
    rec2 = permute(repmat(min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b),[1 p.gridsize_tfp]),[2 1]);
    qD_H = max(0,(1-p.reenterprob)*qD_H1/(1+p.rH) ...
        + p.reenterprob*rec2.*qND_H_haircut);
    qD_L = max(0,(1-p.reenterprob)*qD_L1/(1+p.rL) ...
        + p.reenterprob*rec2.*qND_L_haircut);
    
    defprob = p.prob_y*defprob_new_integrate_eps;
    
    Eqdiff = p.prob_y*qND_diff;
    Eqdiff_writedown = zeros(size(Eqdiff));
    for i = 1:p.gridsize_tfp
        Eqdiff_writedown(i,:) = interp1(p.grid_b,Eqdiff(i,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest');
    end
    ellND = p.prob_liqshock*Eqdiff./qND_H;
    ellD = p.prob_liqshock*((1-p.reenterprob)*p.prob_y*qD_diff ... 
        + p.reenterprob*rec2.*Eqdiff_writedown)./qD_H;

end

function tempdirname = createTempDir

    idx = 1;
    bContinue = true;
    
    while bContinue
        
        tempdirname = ['tempdir',num2str(idx)];
        
        fprintf('Candidate folder name: %s\n',tempdirname);
        
        if ~exist(tempdirname,'dir')
            bContinue = false;
        else
            idx = idx + 1;
        end
        
    end
    
    mkdir(tempdirname);

end