function [VD_new,debt_policy,default_threshold_eps,ceND_new_integrate_eps,...
    qND_H_new_integrate_eps,qND_L_new_integrate_eps,defprob_new_integrate_eps,qND_diff,qD_diff] = ...
    VFI_given_y_b(p,idx_y,idx_b,ceC,ceD,qND_H,qND_L,qD_H,qD_L,defprob)
% NOTE: all inputs, except indices, should be column vectors

    %% update VD
    % ceD(y,b) = theta*E[VD(y',(1-L)*b)|y] + (1-theta)*ceC(y,(1-L)*b)
    VD_new = p.grid_uD_yb(idx_y,idx_b) + p.beta*ceD(idx_b); % ceD does NOT includes discounting by BETA

    %% run the ggq algorithm
    
    clist = p.grid_y_nd(idx_y) - (p.debt_m + (1-p.debt_m)*p.debt_z)*p.grid_b(idx_b) ...
        + qND_H.*(p.grid_b - (1-p.debt_m)*p.grid_b(idx_b)); % function of b'
    Wlist = p.beta*ceC;
    
    idx_keep = find(defprob<=p.max_def_prob | p.grid_b <= (1-p.debt_m)*p.grid_b(idx_b));
    
    if isempty(idx_keep)
        debt_policy.EmptyChoiceSet = true;
        debt_policy.eps_mesh = [p.ggq_mlb p.ggq_mub];
        debt_policy.Vs = NaN;
        debt_policy.NumRegions = 1;
        debt_policy.idx_bnext = NaN;
    else
        temp_debt_policy = ggq_topdown(p.clb,p.rra,p.ggq_mlb,p.ggq_mub,clist(idx_keep),Wlist(idx_keep),p.errtol_bisect);
        if any(isnan(temp_debt_policy.choice_index)) % infeasible consumption choices
            debt_policy.EmptyChoiceSet = true;
            debt_policy.eps_mesh = temp_debt_policy.m_mesh;
            debt_policy.Vs = temp_debt_policy.Vs;
            debt_policy.cs = temp_debt_policy.cs;
            debt_policy.Ws = temp_debt_policy.Ws;
            debt_policy.NumRegions = numel(temp_debt_policy.choice_index);
            debt_policy.idx_bnext = NaN*zeros(size(temp_debt_policy.choice_index));
            for j = 1:debt_policy.NumRegions
                if ~isnan(temp_debt_policy.choice_index(j))
                    debt_policy.idx_bnext(j) = idx_keep(temp_debt_policy.choice_index(j));
                end
            end
        else
            debt_policy.EmptyChoiceSet = false;
            debt_policy.eps_mesh = temp_debt_policy.m_mesh;
            debt_policy.Vs = temp_debt_policy.Vs;
            debt_policy.cs = temp_debt_policy.cs;
            debt_policy.Ws = temp_debt_policy.Ws;
            debt_policy.NumRegions = numel(temp_debt_policy.choice_index);
            debt_policy.idx_bnext = NaN*zeros(size(temp_debt_policy.choice_index));
            for j = 1:debt_policy.NumRegions
                if ~isnan(temp_debt_policy.choice_index(j))
                    debt_policy.idx_bnext(j) = idx_keep(temp_debt_policy.choice_index(j));
                end
            end
        end
    end
    
    if debt_policy.EmptyChoiceSet
        default_threshold_eps = p.ggq_mub;
    else
        if VD_new>=debt_policy.Vs(end)
            % always default
            default_threshold_eps = p.ggq_mub;
        else
            if isnan(debt_policy.cs(1))
                % no choice is infeasible in lowest region
                if VD_new>=debt_policy.Vs(2)
                    idx = find(debt_policy.Vs(1:end-1)<=VD_new & VD_new<=debt_policy.Vs(2:end),1,'first');
                    c = debt_policy.cs(idx);
                    W = debt_policy.Ws(idx);
                    default_threshold_eps = ((VD_new - W)*(1 - p.rra))^(1/(1-p.rra)) - c;
                else
                    default_threshold_eps = debt_policy.m_mesh(2);
                end
            else
                if VD_new>=debt_policy.Vs(1)
                    idx = find(debt_policy.Vs(1:end-1)<=VD_new & VD_new<=debt_policy.Vs(2:end),1,'first');
                    c = debt_policy.cs(idx);
                    W = debt_policy.Ws(idx);
                    default_threshold_eps = ((VD_new - W)*(1 - p.rra))^(1/(1-p.rra)) - c;
                else
                    default_threshold_eps = p.ggq_mlb;
                end
            end
        end
    end
    
    %% partially update value functions and bond prices: integrate out eps
    % ceC: integrate max{VD(y,b),VC(y,eps,b)} over eps
    % qCs: integrate qC(b,bnext(y,eps,b)) over eps
    % need to add more integrations... (1-d)*q, etc...
    
    ceND_new_integrate_eps = 0; % \int VD(y,b)*D(y,e,b)+VC(y,e,b)*(1-D(y,e,b)) de, with (y,b) being parameters
    qND_H_new_integrate_eps = 0; % \int (1-D(y,e,b))*qCL(y,b'(y,e,b)) de, with (y,b) being parameters
    qND_L_new_integrate_eps = 0;
    qND_diff = 0; % E[qH-qL], q=qND, b'=b'(b) when not in default
    qD_diff = qD_H(idx_b) - qD_L(idx_b); % E[qH-qL], q=qD, b'=b
    
    if ~debt_policy.EmptyChoiceSet
        for j = 1:debt_policy.NumRegions
            region_lb = debt_policy.eps_mesh(j);
            region_ub = debt_policy.eps_mesh(j+1);
            % update ceC and qND_H and qND_L
            if default_threshold_eps>=region_ub
                % default in the entire region
                mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                    normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                ceND_new_integrate_eps = ceND_new_integrate_eps + VD_new*mass;
                qND_H_new_integrate_eps = qND_H_new_integrate_eps + ...
                    ((1-p.prob_liqshock)*qD_H(idx_b)...
                     +p.prob_liqshock*qD_L(idx_b))*mass;
                qND_L_new_integrate_eps = qND_L_new_integrate_eps + ...
                    ((1-p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D))*qD_L(idx_b)...
                     +p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D)*qD_H(idx_b))*mass;
                qND_diff = qND_diff + (qD_H(idx_b) - qD_L(idx_b))*mass;
            else
                idx_bnext = debt_policy.idx_bnext(j);
%                 try
                c0 = p.grid_y_nd(idx_y) - (p.debt_m + (1-p.debt_m)*p.debt_z)*p.grid_b(idx_b) ...
                    + qND_H(idx_bnext).*(p.grid_b(idx_bnext) - (1-p.debt_m)*p.grid_b(idx_b)); % function of b'
%                 catch
%                     error('here.');
%                 end
                W0 = ceC(idx_bnext);
                if default_threshold_eps<=region_lb
                    % no default in the entire region
                    % f(Clb,RRA,BETA,C,W)
                    ceND_new_integrate_eps = ceND_new_integrate_eps + ...
                        quadgk(@(eps)normpdf(eps,p.ggq_mmean,p.ggq_mstd).*f(p.clb,p.rra,p.beta,c0+eps,W0),region_lb,region_ub);
                    mass = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    qND_H_new_integrate_eps = qND_H_new_integrate_eps + ...
                        (p.debt_m+(1-p.debt_m)*p.debt_z)*mass...
                        +(1-p.debt_m)*((1-p.prob_liqshock)*qND_H(idx_bnext)...
                        +p.prob_liqshock*qND_L(idx_bnext))*mass;
                    qND_L_new_integrate_eps = qND_L_new_integrate_eps + ...
                        (p.debt_m+(1-p.debt_m)*p.debt_z)*mass...
                        +(1-p.debt_m)*((1-p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND))*qND_L(idx_bnext)...
                         +p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND)*qND_H(idx_bnext))*mass;
                    qND_diff = qND_diff + (1-p.debt_m)*(qND_H(idx_bnext) - qND_L(idx_bnext))*mass;
                else
                    % default then no default
                    ceND_new_integrate_eps = ceND_new_integrate_eps + ...
                        (normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd)-normcdf(region_lb,p.ggq_mmean,p.ggq_mstd))*VD_new + ...
                        quadgk(@(eps)normpdf(eps,p.ggq_mmean,p.ggq_mstd).*f(p.clb,p.rra,p.beta,c0+eps,W0),default_threshold_eps,region_ub);
                    massdef = normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(region_lb,p.ggq_mmean,p.ggq_mstd);
                    massnodef = normcdf(region_ub,p.ggq_mmean,p.ggq_mstd) - ...
                        normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd);
                    qND_H_new_integrate_eps = qND_H_new_integrate_eps + ...
                        (p.debt_m+(1-p.debt_m)*p.debt_z)*massnodef...
                        +(1-p.debt_m)*((1-p.prob_liqshock)*qND_H(idx_bnext)...
                        +p.prob_liqshock*qND_L(idx_bnext))*massnodef...
                        +((1-p.prob_liqshock)*qD_H(idx_b)...
                        +p.prob_liqshock*qD_L(idx_b))*massdef;
                    qND_L_new_integrate_eps = qND_L_new_integrate_eps + ...
                        (p.debt_m+(1-p.debt_m)*p.debt_z)*massnodef...
                        +(1-p.debt_m)*((1-p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND))*qND_L(idx_bnext)...
                         +p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND)*qND_H(idx_bnext))*massnodef...
                         +((1-p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D))*qD_L(idx_b)...
                         +p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D)*qD_H(idx_b))*massdef;
                    qND_diff = qND_diff + (1-p.debt_m)*(qND_H(idx_bnext) - qND_L(idx_bnext))*massnodef ...
                        + (qD_H(idx_b) - qD_L(idx_b))*massdef;
                end
            end
        end
        ceND_new_integrate_eps = ceND_new_integrate_eps/p.ggq_mmass;
        defprob_new_integrate_eps = (normcdf(default_threshold_eps,p.ggq_mmean,p.ggq_mstd) - normcdf(p.ggq_mlb,p.ggq_mmean,p.ggq_mstd))/p.ggq_mmass;
        qND_H_new_integrate_eps = max(0,qND_H_new_integrate_eps)/p.ggq_mmass;
        qND_L_new_integrate_eps = max(0,qND_L_new_integrate_eps - p.holding_cost)/p.ggq_mmass;
        qND_diff = qND_diff/p.ggq_mmass;
    else
        % debt_policy.EmptyChoiceSet = true.
        % so default throughout
        ceND_new_integrate_eps = VD_new;
        defprob_new_integrate_eps = 1;
        qND_H_new_integrate_eps = max(0,(1-p.prob_liqshock)*qD_H(idx_b)...
                        +p.prob_liqshock*qD_L(idx_b));
        qND_L_new_integrate_eps = max(0,(1-p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D))*qD_L(idx_b)...
                        +p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D)*qD_H(idx_b) - p.holding_cost);
        qND_diff = qD_H(idx_b) - qD_L(idx_b);
    end    

end

%% valuation
function V = f(Clb,RRA,BETA,C,W)
    
    V = max(Clb,C).^(1-RRA)/(1-RRA) + BETA*W;

end