set(0,'DefaultFigureWindowStyle','docked')
clear all

load('baseline_11042020.mat','p','model')

%% plot bond prices

LineWidth = 2;
FontSize = 18;
LegendFontSize = 18;

% STD_DEV = 2;
STD_DEV = 1;

[~,idxH] = min(abs(log(p.grid_y_nd) - STD_DEV*p.sigma_tfp/sqrt(1-p.rho_tfp^2)));
[~,idxL] = min(abs(log(p.grid_y_nd) + STD_DEV*p.sigma_tfp/sqrt(1-p.rho_tfp^2)));

figure

subplot(1,3,1)
hold on
plot(p.grid_b,model.qNDH(idxH,:),'-b','LineWidth',LineWidth)
plot(p.grid_b,model.qNDH(idxL,:),'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 1.205])
title('A. Bond price schedule, $q^H_{ND}$','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt choice, $b''$','Interpreter','Latex','FontSize',FontSize)
h = legend({['High',char(10),'output'],['Low',char(10),'output']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','SouthWest');
rect = [0.06, 0.3, .25, .25];
set(h, 'Position', rect)
grid on
legend('boxoff')

subplot(1,3,2)
hold on
qsaleH = p.bargain_power_mkt_maker_ND*model.qNDL(idxH,:) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxH,:);
baH = 2*(model.qNDH(idxH,:) - qsaleH)./(model.qNDH(idxH,:) + qsaleH);
plot(p.grid_b,baH,'-b','LineWidth',LineWidth)
qsaleL = p.bargain_power_mkt_maker_ND*model.qNDL(idxL,:) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxL,:);
baL = 2*(model.qNDH(idxL,:) - qsaleL)./(model.qNDH(idxL,:) + qsaleL);
plot(p.grid_b,baL,'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt choice, $b''$','Interpreter','Latex','FontSize',FontSize)
legend({'High output','Low output'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest');
title('B. Bid-ask, not in default','Interpreter','Latex','FontSize',FontSize)
grid on
legend('boxoff')

subplot(1,3,3)
hold on
qsaleH = p.bargain_power_mkt_maker_ND*model.qDL(idxH,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxH,:);
baH = 2*(model.qDH(idxH,:) - qsaleH)./(model.qDH(idxH,:) + qsaleH);
plot(p.grid_b,baH,'-b','LineWidth',LineWidth)
qsaleL = p.bargain_power_mkt_maker_ND*model.qDL(idxL,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxL,:);
baL = 2*(model.qDH(idxL,:) - qsaleL)./(model.qDH(idxL,:) + qsaleL);
plot(p.grid_b,baL,'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt (defaulted), $b$','Interpreter','Latex','FontSize',FontSize)
legend({'High output','Low output'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
title('C. Bid-ask, in default','Interpreter','Latex','FontSize',FontSize)
grid on
legend('boxoff')

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_bond_prices_and_bid_ask_1x3.eps');

%% bond price, version 2

figure

m0 = p.ggq_mmean;
% m0 = p.ggq_mlb;
% m0 = p.ggq_mub;
policy_idxH = zeros(size(p.grid_b));
policy_idxL = zeros(size(p.grid_b));
for i = 1:p.gridsize_b
    idx = find(model.policy{idxH,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idxH,i}.eps_mesh(2:end),1,'first');
    policy_idxH(i) = model.policy{idxH,i}.idx_bnext(idx);
    idx = find(model.policy{idxL,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idxL,i}.eps_mesh(2:end),1,'first');
    policy_idxL(i) = model.policy{idxL,i}.idx_bnext(idx);
end

subplot(1,3,1)
hold on
plot(p.grid_b,model.qNDH(idxH,policy_idxH),'-b','LineWidth',LineWidth)
plot(p.grid_b,model.qNDH(idxL,policy_idxL),'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 1.205])
title('A. Bond price, $q^H_{ND}$','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b$','Interpreter','Latex','FontSize',FontSize)
h = legend({['High',char(10),'output'],['Low',char(10),'output']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','SouthWest');
rect = [0.06, 0.3, .25, .25];
set(h, 'Position', rect)
grid on
legend('boxoff')

subplot(1,3,2)
hold on
qsaleH = p.bargain_power_mkt_maker_ND*model.qNDL(idxH,policy_idxH) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxH,policy_idxH);
baH = 2*(model.qNDH(idxH,policy_idxH) - qsaleH)./(model.qNDH(idxH,policy_idxH) + qsaleH);
plot(p.grid_b,baH,'-b','LineWidth',LineWidth)
qsaleL = p.bargain_power_mkt_maker_ND*model.qNDL(idxL,policy_idxL) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxL,policy_idxL);
baL = 2*(model.qNDH(idxL,policy_idxL) - qsaleL)./(model.qNDH(idxL,policy_idxL) + qsaleL);
plot(p.grid_b,baL,'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt level, $b$','Interpreter','Latex','FontSize',FontSize)
legend({'High output','Low output'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest');
title('B. Bid-ask, not in default','Interpreter','Latex','FontSize',FontSize)
grid on
legend('boxoff')

subplot(1,3,3)
hold on
qsaleH = p.bargain_power_mkt_maker_ND*model.qDL(idxH,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxH,:);
baH = 2*(model.qDH(idxH,:) - qsaleH)./(model.qDH(idxH,:) + qsaleH);
plot(p.grid_b,baH,'-b','LineWidth',LineWidth)
qsaleL = p.bargain_power_mkt_maker_ND*model.qDL(idxL,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxL,:);
baL = 2*(model.qDH(idxL,:) - qsaleL)./(model.qDH(idxL,:) + qsaleL);
plot(p.grid_b,baL,'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt (defaulted), $b$','Interpreter','Latex','FontSize',FontSize)
legend({'High output','Low output'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
title('C. Bid-ask, in default','Interpreter','Latex','FontSize',FontSize)
grid on
legend('boxoff')

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_bond_prices_and_bid_ask_1x3_v2.eps');

%% bond price, version 3

figure

m0 = p.ggq_mmean;
% m0 = p.ggq_mlb;
% m0 = p.ggq_mub;
policy_idxH = zeros(size(p.grid_b));
policy_idxL = zeros(size(p.grid_b));
qH = NaN*ones(size(p.grid_b));
qL = NaN*ones(size(p.grid_b));
baH = NaN*ones(size(p.grid_b));
baL = NaN*ones(size(p.grid_b));

for i = 1:p.gridsize_b
    idx = find(model.policy{idxH,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idxH,i}.eps_mesh(2:end),1,'first');
    policy_idxH(i) = model.policy{idxH,i}.idx_bnext(idx);
    if model.defthresh(idxH,i)<m0
        qH(i) = model.qNDH(idxH,policy_idxH(i));
        qsaleH = p.bargain_power_mkt_maker_ND*model.qNDL(idxH,policy_idxH(i)) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxH,policy_idxH(i));
        baH(i) = 2*(model.qNDH(idxH,policy_idxH(i)) - qsaleH)./(model.qNDH(idxH,policy_idxH(i)) + qsaleH);
    end
    idx = find(model.policy{idxL,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idxL,i}.eps_mesh(2:end),1,'first');
    policy_idxL(i) = model.policy{idxL,i}.idx_bnext(idx);
    if model.defthresh(idxL,i)<m0
        qL(i) = model.qNDH(idxL,policy_idxL(i));
        qsaleL = p.bargain_power_mkt_maker_ND*model.qNDL(idxL,policy_idxL(i)) + (1-p.bargain_power_mkt_maker_ND)*model.qNDH(idxL,policy_idxL(i));
        baL(i) = 2*(model.qNDH(idxL,policy_idxL(i)) - qsaleL)./(model.qNDH(idxL,policy_idxL(i)) + qsaleL);
    end
end

subplot(1,3,1)
hold on
plot(p.grid_b/3,qH,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,qL,'-.r','LineWidth',LineWidth)
axis square
box on
% set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 1.205])
title('A. Bond price, $q^H_{ND}$','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
h = legend({['High',char(10),'output'],['Low',char(10),'output']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','SouthWest');
% rect = [0.06, 0.3, .25, .25];
% set(h, 'Position', rect)
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.65])
set(gca,'xtick',0:.4:1.6)
set(gca,'xticklabel',0:.4:1.6)
set(gca,'ylim',[0.2 1.2])
set(gca,'ytick',0.2:0.2:1.2)
set(gca,'yticklabel',0.2:0.2:1.2)

subplot(1,3,2)
hold on
plot(p.grid_b/3,baH,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,baL,'-.r','LineWidth',LineWidth)
axis square
box on
% set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
h = legend({['High',char(10),'output'],['Low',char(10),'output']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest');
title('B. Bid-ask, not in default','Interpreter','Latex','FontSize',FontSize)
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.65])
set(gca,'xtick',0:.4:1.6)
set(gca,'xticklabel',0:.4:1.6)
% set(gca,'yticklabel',[0.004:0.002:0.014])

subplot(1,3,3)
hold on
qsaleH = p.bargain_power_mkt_maker_ND*model.qDL(idxH,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxH,:);
baH = 2*(model.qDH(idxH,:) - qsaleH)./(model.qDH(idxH,:) + qsaleH);
plot(p.grid_b/3,baH,'-b','LineWidth',LineWidth)
qsaleL = p.bargain_power_mkt_maker_ND*model.qDL(idxL,:) + (1-p.bargain_power_mkt_maker_ND)*model.qDH(idxL,:);
baL = 2*(model.qDH(idxL,:) - qsaleL)./(model.qDH(idxL,:) + qsaleL);
plot(p.grid_b/3,baL,'-.r','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.08])
xlabel('Debt level (in default), $b/3$','Interpreter','Latex','FontSize',FontSize)
h = legend({['High',char(10),'output'],['Low',char(10),'output']},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest');
title('C. Bid-ask, in default','Interpreter','Latex','FontSize',FontSize)
% grid on
legend('boxoff')
set(gca,'xlim',[0 1.65])
set(gca,'xtick',0:.4:1.6)
set(gca,'xticklabel',0:.4:1.6)
% set(gca,'ylim',[0 0.08])
% set(gca,'ytick',0:.02:0.08)
% set(gca,'yticklabel',0:.02:0.08)

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_bond_prices_and_bid_ask_1x3_v3.eps');

%% spread decomposition, v1

figure

subplot(1,2,1)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qNDH(idxL,:) - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qND_noliq(idxL,:) - p.debt_m).^12 - (1 + p.rH).^12;
% idx = find(spreads1>max_spread);
idx = find(model.defprob(idxL,:)>p.max_def_prob);
spreads1(idx) = NaN;
spreads2(idx) = NaN;
plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
% axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.2])
% set(gca,'ylim',[0 0.3])
set(gca,'ylim',[0 0.5])
set(gca,'ytick',[0:0.1:0.5])
set(gca,'yticklabel',[0:0.1:0.5])
title('A. Low output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt choice, $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')

subplot(1,2,2)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qNDH(idxH,:) - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./model.qND_noliq(idxH,:) - p.debt_m).^12 - (1 + p.rH).^12;
% idx = find(spreads1>max_spread);
idx = find(model.defprob(idxH,:)>p.max_def_prob);
spreads1(idx) = NaN;
spreads2(idx) = NaN;
plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
% axis square
box on
set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'ylim',[0 0.2])
% set(gca,'ylim',[0 0.3])
set(gca,'ylim',[0 0.5])
set(gca,'ytick',[0:0.1:0.5])
set(gca,'yticklabel',[0:0.1:0.5])
title('B. High output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt choice, $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_spreads_decomposition_2term_1x2_v1.eps');

%% baseline spread decomposition, v2

m0 = p.ggq_mmean;

figure

idx = idxL;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext1 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext1(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
%     else
%         qchoice_NDH(i) = model.qDH(idx,i);
%         qchoice_noliq(i) = model.qD_noliq(idx,i);
    end
end

subplot(1,2,1)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
axis square
box on
% set(gca,'xlim',p.grid_b([1 end]))
set(gca,'xlim',[0 5])
% set(gca,'ylim',[0 0.25])
set(gca,'ylim',[0 0.3])
title('A. Low output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')

idx = idxH;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext2 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext2(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
    end
end

subplot(1,2,2)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
axis square
box on
set(gca,'xlim',[0 5])
% set(gca,'ylim',[0 0.25])
set(gca,'ylim',[0 0.3])
title('B. High output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_spreads_decomposition_2term_1x2_v2.eps');

%% plot for relevant range only

m0 = p.ggq_mmean;

figure

idx = idxL;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext1 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext1(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
%     else
%         qchoice_NDH(i) = model.qDH(idx,i);
%         qchoice_noliq(i) = model.qD_noliq(idx,i);
    end
end

subplot(1,2,1)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
plot(p.grid_b/3,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b/3,spreads1 - spreads2,'--k','LineWidth',LineWidth)
% axis square
box on
% set(gca,'xlim',p.grid_b([1 end]))
set(gca,'xlim',[0 1.65])
set(gca,'xtick',0:.4:1.6)
set(gca,'xticklabel',0:.4:1.6)
% set(gca,'ylim',[0 0.25])
set(gca,'ylim',[0 0.5])
set(gca,'ytick',0:0.1:0.5)
set(gca,'yticklabel',0:0.1:0.5)
title('A. Low output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
% grid on
legend('boxoff')

idx = idxH;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext2 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext2(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
    end
end

subplot(1,2,2)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
plot(p.grid_b/3,spreads1,'-b','LineWidth',LineWidth)
plot(p.grid_b/3,spreads2,'-.r','LineWidth',LineWidth)
plot(p.grid_b/3,spreads1 - spreads2,'--k','LineWidth',LineWidth)
% axis square
box on
set(gca,'xlim',[0 1.65])
set(gca,'xtick',0:.4:1.6)
set(gca,'xticklabel',0:.4:1.6)
% set(gca,'ylim',[0 0.25])
set(gca,'ylim',[0 0.5])
set(gca,'ytick',0:0.1:0.5)
set(gca,'yticklabel',0:0.1:0.5)
title('B. High output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Spreads (annualized)','Interpreter','Latex','FontSize',FontSize)
legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
% grid on
legend('boxoff')

h=gcf;
set(h,'PaperOrientation','landscape');
set(h,'PaperUnits','normalized');
set(h,'PaperPosition', [0 0 1 1]);
% print(gcf, '-depsc2', 'baseline_spreads_decomposition_2term_1x2_v3.eps');

%% fractions

m0 = p.ggq_mmean;

figure

idx = idxL;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext1 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext1(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
%     else
%         qchoice_NDH(i) = model.qDH(idx,i);
%         qchoice_noliq(i) = model.qD_noliq(idx,i);
    end
end

subplot(1,2,1)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
% plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
% plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
% plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
plot(p.grid_b/3,(spreads1 - spreads2)./spreads1,'--k','LineWidth',LineWidth)
axis square
box on
% set(gca,'xlim',p.grid_b([1 end]))
% set(gca,'xlim',[0 5])
% set(gca,'ylim',[0 0.25])
% set(gca,'ylim',[0 0.5])
title('A. Low output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Frac spreads due to liq.','Interpreter','Latex','FontSize',FontSize)
% legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')

idx = idxH;
qchoice_NDH = NaN*zeros(size(model.qNDH(idx,:)));
qchoice_noliq = NaN*zeros(size(model.qNDH(idx,:)));
bnext2 = NaN*zeros(size(model.qNDH(idx,:)));

for i = 1:p.gridsize_b
    if model.defthresh(idx,i)<m0 && ~model.policy{idx,i}.EmptyChoiceSet
        temp_idx = find(model.policy{idx,i}.eps_mesh(1:end-1)<=m0 & m0<=model.policy{idx,i}.eps_mesh(2:end),1,'first');
        qchoice_NDH(i) = model.qNDH(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        qchoice_noliq(i) = model.qND_noliq(idx,model.policy{idx,i}.idx_bnext(temp_idx));
        bnext2(i) = p.grid_b(model.policy{idx,i}.idx_bnext(temp_idx));
    end
end

subplot(1,2,2)
hold on
spreads1 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_NDH - p.debt_m).^12 - (1 + p.rH).^12;
spreads2 = (1 + (p.debt_m + (1-p.debt_m)*p.debt_z)./qchoice_noliq - p.debt_m).^12 - (1 + p.rH).^12;
% plot(p.grid_b,spreads1,'-b','LineWidth',LineWidth)
% plot(p.grid_b,spreads2,'-.r','LineWidth',LineWidth)
% plot(p.grid_b,spreads1 - spreads2,'--k','LineWidth',LineWidth)
plot(p.grid_b/3,(spreads1 - spreads2)./spreads1,'--k','LineWidth',LineWidth)
axis square
box on
% set(gca,'xlim',[0 5])
% set(gca,'ylim',[0 0.25])
% set(gca,'ylim',[0 0.5])
title('B. High output','Interpreter','Latex','FontSize',FontSize)
xlabel('Debt level, $b/3$','Interpreter','Latex','FontSize',FontSize)
% ylabel('Spreads (annualized), optimal $b''$','Interpreter','Latex','FontSize',FontSize)
ylabel('Frac spreads due to liq.','Interpreter','Latex','FontSize',FontSize)
% legend({'Total','Default','Liquidity'},'Interpreter','Latex','FontSize',LegendFontSize,'Location','NorthWest')
grid on
legend('boxoff')