addpath(genpath('../matlab'))
addpath('../src_mex')
clear all

%% load information

load('baseline_no_liq_11042020.mat','p_no_liq','model_no_liq');

load('baseline_11042020.mat','p','model');

model.qND_pureDEF = model_no_liq.qNDH;
model.qD_pureDEF = model_no_liq.qDH;

%% bond prices

% [qND_noliq,qD_noliq] = CalcBondPrices_noliq(p,default_threshold_eps,debt_policy)

tic
[qND_b0_d1_noliq,qD_b0_d1_noliq] = CalcBondPrices_noliq(p_no_liq,model.defthresh,model_no_liq.policy);
toc
[qND_b1_d0_noliq,qD_b1_d0_noliq] = CalcBondPrices_noliq(p_no_liq,model_no_liq.defthresh,model.policy);
toc

model.qND_b0_d1_noliq = qND_b0_d1_noliq;
model.qD_b0_d1_noliq = qD_b0_d1_noliq;
model.qND_b1_d0_noliq = qND_b1_d0_noliq;
model.qD_b1_d0_noliq = qD_b1_d0_noliq;

% compute pure liquidity price
btemp = [p.debt_m + (1-p.debt_m)*p.debt_z;p.debt_m + (1-p.debt_m)*p.debt_z-p.holding_cost];
Atemp = [1+p.rH-(1-p.debt_m)*(1-p.prob_liqshock),-(1-p.debt_m)*p.prob_liqshock;...
    -(1-p.debt_m)*p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND),1+p.rL-(1-p.debt_m)*(1-p.prob_meet_mkt_maker_ND*(1-p.bargain_power_mkt_maker_ND))];
qs_pureLIQ = Atemp\btemp;

model.qH_pureLIQ = qs_pureLIQ(1);
model.qL_pureLIQ = qs_pureLIQ(2);

%% simulate

TBurn = 12*10^3;
T = TBurn + 12*10^6;
QuarterLength = 3;
NumQuartersExcludeAfterReentry = 20;

rng(0);
Us_z = rand(T,1);
Us_m = rand(T,1);
Us_reenter = rand(T,1);

y_pctile = 100*1/9; % last year of the 1993-2001 period
% y_pctile = 25;
% y_pctile = 75;

tic
stats = SimSinglePathDecompose6(p,model,T,TBurn,Us_z,Us_m,Us_reenter,QuarterLength,NumQuartersExcludeAfterReentry,y_pctile);
toc

%% save

save([mfilename,'_p',num2str(y_pctile),'.mat'],'stats');

%% print

fprintf('\n\n')

NameList = {'Total spreads, $cs$',...
    'Default component, $cs_{DEF}$',...
    '\quad Pure default, $cs_{pureDEF}$',...
    '\quad Liquidity driven default, $cs_{LIQ\rightarrow DEF}$',...
    '\qquad Through debt policy, $cs_{LIQ\rightarrow DEF, Debt}$',...
    '\qquad Through default policy, $cs_{LIQ\rightarrow DEF, Def}$',...
    'Liquidity component, $cs_{LIQ}$', ...
    '\quad Pure liquidity, $cs_{pureLIQ}$',...
    '\quad Default driven liquidity, $cs_{DEF\rightarrow LIQ}$'};
FieldList = {'Spread','SpreadDEF','Spread_pureDEF','Spread_LIQDEF',...
    'Spread_LIQDEF_DEBTPOL', 'Spread_LIQDEF_DEFPOL', 'SpreadLIQ',...
    'Spread_pureLIQ', 'Spread_DEFLIQ'};

for i = 1:numel(NameList)
    if strcmpi(FieldList{i},'Spread_pureLIQ')
        rowvals(1) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(2) = 0.0000;
        rowvals(3) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(4) = 0.0000;
        rowvals(5) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(6) = 0.0000;
    else
        rowvals(1) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(2) = getfield(stats,[FieldList{i},'_std']);
        rowvals(3) = getfield(stats,[FieldList{i},'_H_output_mean']);
        rowvals(4) = getfield(stats,[FieldList{i},'_H_output_std']);
        rowvals(5) = getfield(stats,[FieldList{i},'_L_output_mean']);
        rowvals(6) = getfield(stats,[FieldList{i},'_L_output_std']);
    end
    fprintf('%s & %.4f & %.4f && %.4f & %.4f && %.4f & %.4f\\\\\n',...
        NameList{i}, rowvals)
end

fprintf('\n\n')

%% print, mean only

fprintf('\n\n')

NameList = {'Total spreads, $cs$',...
    'Default component, $cs_{DEF}$',...
    '\quad Pure default, $cs_{pureDEF}$',...
    '\quad Liquidity driven default, $cs_{LIQ\rightarrow DEF}$',...
    '\qquad Through debt policy, $cs_{LIQ\rightarrow DEF, Debt}$',...
    '\qquad Through default policy, $cs_{LIQ\rightarrow DEF, Def}$',...
    'Liquidity component, $cs_{LIQ}$', ...
    '\quad Pure liquidity, $cs_{pureLIQ}$',...
    '\quad Default driven liquidity, $cs_{DEF\rightarrow LIQ}$'};
FieldList = {'Spread','SpreadDEF','Spread_pureDEF','Spread_LIQDEF',...
    'Spread_LIQDEF_DEBTPOL', 'Spread_LIQDEF_DEFPOL', 'SpreadLIQ',...
    'Spread_pureLIQ', 'Spread_DEFLIQ'};

for i = 1:numel(NameList)
    if strcmpi(FieldList{i},'Spread_pureLIQ')
        rowvals(1) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(2) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(3) = getfield(stats,[FieldList{i},'_mean']);
    else
        rowvals(1) = getfield(stats,[FieldList{i},'_mean']);
        rowvals(2) = getfield(stats,[FieldList{i},'_H_output_mean']);
        rowvals(3) = getfield(stats,[FieldList{i},'_L_output_mean']);
    end
    fprintf('%s & %.4f & %.4f & %.4f \\\\\n',...
        NameList{i}, rowvals)
end

fprintf('\n\n')