#include "realtype.h"
#include "ggq.h"
#include "liquidity_defs.h"
#include "normaldist.h"
#include <math_constants.h>
#include <cstdio>

// #define DEBUG_PRINTF

__device__ void ggq_topdown(REAL_TYPE &CENDupdate, REAL_TYPE &qHupdate, REAL_TYPE &qLupdate, REAL_TYPE &def_prob_update, REAL_TYPE &mdeftresh,
	int idx_y, int idx_b, REAL_TYPE &VD, 
	REAL_TYPE *d_qH_ND, REAL_TYPE *d_qL_ND, REAL_TYPE *d_qH_D, REAL_TYPE *d_qL_D, REAL_TYPE *d_CVND, REAL_TYPE *d_def_prob)
{

	// initialize at mub
	REAL_TYPE Vnow = CUDART_NEG_INF;
	REAL_TYPE Cnow, Wnow;
	REAL_TYPE Ccandidate, Wcandidate, Vcandidate;
	int i, ichoice;
	REAL_TYPE mnow = GGQ_MUB;

	// initialize at m1
	for (i = 0; i < GRIDSIZE_B; i++)
	{
		// cs0 = y - b*[m + (1 - m)*z] + qCH(y, b')*[b' - (1 - m)*b]
		// NOTE: we require cs0+mlb to be strictly positive!
		Ccandidate = GRID_Y_ND[idx_y] - GRID_B[idx_b] * (DEBT_M + (1 - DEBT_M)*DEBT_Z)
		+ d_qH_ND[idx_y*GRIDSIZE_B + i] * (GRID_B[i] - (1 - DEBT_M)*GRID_B[idx_b]);
		if (Ccandidate + mnow >= C_LB &&
			(d_def_prob[idx_y*GRIDSIZE_B + i]<=MAX_DEF_PROB || GRID_B[i]<=(1 - DEBT_M)*GRID_B[idx_b]))
		{
			Wcandidate = BETA*d_CVND[idx_y*GRIDSIZE_B + i];
			Vcandidate = U_SCALING*POWERFUN(Ccandidate + mnow, ONE_MINUS_RRA) + Wcandidate; //mnow=GGQ_MLB
			// U_SCALING = 1/(1-RRA)
			if (Vcandidate > Vnow)
			{
				Cnow = Ccandidate;
				Wnow = Wcandidate;
				Vnow = Vcandidate;
				// record choices
				ichoice = i;
			}
			else if (Vcandidate == Vnow)
			{
				if (Ccandidate < Cnow)//choose lowest c
				{
					Cnow = Ccandidate;
					Wnow = Wcandidate;
					Vnow = Vcandidate;
					// record choices
					ichoice = i;
				}
			}
		}
	} // initialize

	if (!isfinite(Vnow))
	{
		// // no feasible choice; always default
		CENDupdate = VD;
		def_prob_update = 1.0;
		#ifdef DEBUG_PRINTF
			if (idx_y==41 && idx_b==215)
			{
			printf("A. q'=%g\n",
				(1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b]);
			}
		#endif
		qHupdate = (1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b];
		qLupdate = max(0.0,(1 - PROB_LH_D)*d_qL_D[idx_y*GRIDSIZE_B + idx_b] + PROB_LH_D*d_qH_D[idx_y*GRIDSIZE_B + idx_b]
			 - HOLDING_COST);
		mdeftresh = GGQ_MUB;
	}
	else
	{

		int ichoice_prev = ichoice;
		REAL_TYPE M1;
		REAL_TYPE Vnow_M1;
		REAL_TYPE mnext, Cnext, Wnext;
		bool bContinue = true;

		CENDupdate = 0.0, qHupdate = 0.0, qLupdate = 0.0;

		if (VD>=Vnow)
		{
			// always default
			mdeftresh = GGQ_MUB;
		}
		else
		{
			mdeftresh = CUDART_NEG_INF; // infinite number will be used as a flag
		}

		while (bContinue)
		{

			M1 = max(GGQ_MLB, C_LB - Cnow);
			Vnow_M1 = U_SCALING*POWERFUN(Cnow + M1, ONE_MINUS_RRA) + Wnow;
			mnext = M1;
			Cnext = Cnow;
			bContinue = false;

			// go through choices
			for (i = 0; i < GRIDSIZE_B; i++)
			{
				if (i != ichoice_prev &&
					(d_def_prob[idx_y*GRIDSIZE_B + i]<=MAX_DEF_PROB || GRID_B[i]<=(1 - DEBT_M)*GRID_B[idx_b]))
				{
					Ccandidate = GRID_Y_ND[idx_y] - GRID_B[idx_b] * (DEBT_M + (1 - DEBT_M)*DEBT_Z)
						+ d_qH_ND[idx_y*GRIDSIZE_B + i] * (GRID_B[i] - (1 - DEBT_M)*GRID_B[idx_b]);
					if (Ccandidate>Cnow) // only larger c is possibly a better choice, feasibility is automatically satisfied!
					{
						Wcandidate = BETA*d_CVND[idx_y*GRIDSIZE_B + i];
						Vcandidate = U_SCALING*POWERFUN(Ccandidate + M1, ONE_MINUS_RRA) + Wcandidate;
						if (Vcandidate>Vnow_M1)
						{
							// find bisection point, save a register-- store in Vcandidate
							Vcandidate = bisect_zero(M1, mnow, Cnow, Ccandidate, (Wnow - Wcandidate) / U_SCALING);
							if (Vcandidate > mnext)
							{
								// NOTE: if the difference is too small, the root from the bisection may be equal to MUB. This will produce an error.
								ichoice = i;
								mnext = Vcandidate;
								Cnext = Ccandidate;
								Wnext = Wcandidate;
								bContinue = true;
							}
							else if (Vcandidate == mnext && Ccandidate<Cnext) // choose lowest consumption
							{
								ichoice = i;
								mnext = Vcandidate;
								Cnext = Ccandidate;
								Wnext = Wcandidate;
								bContinue = true;
							}
						} // is a better choice
					} // possible better choice
				}
			} // note, it's possible for this loop to have no feasible choice (e.g. q=0)

			if (VD >= U_SCALING*POWERFUN(Cnow + mnow, ONE_MINUS_RRA) + Wnow)
			{
                // always default in region
                CENDupdate += VD*(normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD));
				#ifdef DEBUG_PRINTF
				if (idx_y==41 && idx_b==215)
				{
					printf("B. prob=%g,q'=%g,mass=%g,P=%g,q1=%g, q2=%g\n",normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD),
						(1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b], GGQ_MMASS,
						PROB_HL, d_qH_D[idx_y*GRIDSIZE_B + idx_b], d_qL_D[idx_y*GRIDSIZE_B + idx_b]);
				}
				#endif
				qHupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
					((1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b]);
				qLupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
					((1 - PROB_LH_D)*d_qL_D[idx_y*GRIDSIZE_B + idx_b] + PROB_LH_D*d_qH_D[idx_y*GRIDSIZE_B + idx_b]);
                bContinue = false;
			}
			else if (VD<=U_SCALING*POWERFUN(Cnow + mnext, ONE_MINUS_RRA) + Wnow)
			{
                // never default in region
                if (bContinue)
                {
                    CENDupdate += gauss_legendre_CENDupdate(mnext, mnow, Cnow, Wnow);
                    qHupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(mnext - GGQ_MMEAN, GGQ_MSTD))*
						(DEBT_M + (1-DEBT_M)*DEBT_Z 
							+ (1-DEBT_M)*((1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev])
							);
					#ifdef DEBUG_PRINTF
					if (idx_y==41 && idx_b==215)
					{
						printf("C. prob=%g,q'=%g,mass=%g,P=%g,q1=%g, q2=%g\n",normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(mnext - GGQ_MMEAN, GGQ_MSTD),
							(1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev], GGQ_MMASS,
							PROB_HL, d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev], d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev]);
					}
					#endif
					qLupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(mnext - GGQ_MMEAN, GGQ_MSTD))*
						(DEBT_M + (1 - DEBT_M)*DEBT_Z 
							+ (1 - DEBT_M)*((1 - PROB_LH_ND)*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_LH_ND*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev])
							);
                    // update and continue algorithm
					ichoice_prev = ichoice;
					mnow = mnext;
					Cnow = Cnext;
					Wnow = Wnext;
                }
                else
                {
                    if (C_LB - Cnow > GGQ_MLB)
                    {
                        // (Cnow,Wnow) applies until M1=C_LB-Cnow>MLB
                        CENDupdate += gauss_legendre_CENDupdate(M1, mnow, Cnow, Wnow);
						qHupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(M1 - GGQ_MMEAN, GGQ_MSTD))*
							(DEBT_M + (1-DEBT_M)*DEBT_Z 
								+ (1-DEBT_M)*((1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev])
								);
						#ifdef DEBUG_PRINTF
						if (idx_y==41 && idx_b==215)
						{
							printf("D. prob=%g,q'=%g,mass=%g,P=%g,q1=%g, q2=%g\n",normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(M1 - GGQ_MMEAN, GGQ_MSTD),
								(1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev], GGQ_MMASS,
								PROB_HL, d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev], d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev]);
						}
						#endif
						qLupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(M1 - GGQ_MMEAN, GGQ_MSTD))*
							(DEBT_M + (1 - DEBT_M)*DEBT_Z 
								+ (1 - DEBT_M)*((1 - PROB_LH_ND)*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_LH_ND*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev])
								);
                        // reinitialize algorithm at the maximal value at M1
                        bContinue = false;
                        mnow = M1;
                        Vnow = Vnow_M1;
                        for (i = 0; i < GRIDSIZE_B; i++)
                        {
                            if (i!=ichoice &&
								(d_def_prob[idx_y*GRIDSIZE_B + i]<=MAX_DEF_PROB || GRID_B[i]<=(1 - DEBT_M)*GRID_B[idx_b]))
                            {
                                Ccandidate = GRID_Y_ND[idx_y] - GRID_B[idx_b] * (DEBT_M + (1 - DEBT_M)*DEBT_Z)
                                    + d_qH_ND[idx_y*GRIDSIZE_B + i] * (GRID_B[i] - (1 - DEBT_M)*GRID_B[idx_b]);
                                if (Ccandidate + M1 > C_LB)
                                {
                                    Wcandidate = BETA*d_CVND[idx_y*GRIDSIZE_B + i];
                                    Vcandidate = U_SCALING*POWERFUN(Ccandidate + M1, ONE_MINUS_RRA) + Wcandidate;
                                    if (Vcandidate > Vnow)
                                    {
                                        Cnow = Ccandidate;
                                        Wnow = Wcandidate;
                                        Vnow = Vcandidate;
                                        // record choices
                                        ichoice_prev = i;
                                        bContinue = true;
                                    }
                                    else if (Vcandidate == Vnow)
                                    {
                                        if (Ccandidate < Cnow)
                                        {
                                            Cnow = Ccandidate;
                                            Wnow = Wcandidate;
                                            Vnow = Vcandidate;
                                            // record choices
                                            ichoice_prev = i;
                                            bContinue = true;
                                        }
                                    }
                                }
                            }
                        }
                        if (!bContinue)
                        {
                            // all choices are not feasible, VD applies for m<=M1
                            CENDupdate += VD*(normcdf(M1 - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD));
							qHupdate += (normcdf(M1 - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
								((1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b]);
							#ifdef DEBUG_PRINTF
							if (idx_y==41 && idx_b==215)
							{
								printf("E. prob=%g,q'=%g,P=%g,q1=%g, q2=%g\n",normcdf(M1 - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD),
									(1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b],
									PROB_HL, d_qH_D[idx_y*GRIDSIZE_B + idx_b], d_qL_D[idx_y*GRIDSIZE_B + idx_b]);
							}
							#endif
							qLupdate += (normcdf(M1 - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
								((1 - PROB_LH_D)*d_qL_D[idx_y*GRIDSIZE_B + idx_b] + PROB_LH_D*d_qH_D[idx_y*GRIDSIZE_B + idx_b]);
							mdeftresh = M1;
                        }
                    }
                    else
                    {
                        // (Cnow,Wnow) applies until MLB
                        CENDupdate += gauss_legendre_CENDupdate(GGQ_MLB, mnow, Cnow, Wnow);
						qHupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
							(DEBT_M + (1-DEBT_M)*DEBT_Z 
								+ (1-DEBT_M)*((1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev])
								);
						#ifdef DEBUG_PRINTF
						if (idx_y==41 && idx_b==215)
						{
							printf("F. prob=%g,q'=%g,mass=%g,P=%g,q1=%g, q2=%g\n",normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD),
								(1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev], GGQ_MMASS,
								PROB_HL, d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev], d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev]);
						}
						#endif
						qLupdate += (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
							(DEBT_M + (1 - DEBT_M)*DEBT_Z 
								+ (1 - DEBT_M)*((1 - PROB_LH_ND)*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_LH_ND*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev])
								);
						mdeftresh = GGQ_MLB;
                    }
                }
			}
			else
			{
                // intermediate default
                Vcandidate = POWERFUN(ONE_MINUS_RRA*(VD - Wnow),U_SCALING) - Cnow; // def threshold
                CENDupdate += gauss_legendre_CENDupdate(Vcandidate, mnow, Cnow, Wnow)
                    + VD*(normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD));
				qHupdate += (normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
					((1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b])
					+ (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD))*
					(DEBT_M + (1 - DEBT_M)*DEBT_Z + (1 - DEBT_M)*((1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev]));
				#ifdef DEBUG_PRINTF
				if (idx_y==41 && idx_b==215)
				{
					printf("G. prob=%g,q'=%g,mass=%g,q2=%g\n",normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD) - normcdf(mnext - GGQ_MMEAN, GGQ_MSTD),
						(1 - PROB_HL)*d_qH_D[idx_y*GRIDSIZE_B + idx_b] + PROB_HL*d_qL_D[idx_y*GRIDSIZE_B + idx_b], 
						(1 - PROB_HL)*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_HL*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev]);
				}
				#endif
				qLupdate += (normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))*
					((1 - PROB_LH_D)*d_qL_D[idx_y*GRIDSIZE_B + idx_b] + PROB_LH_D*d_qH_D[idx_y*GRIDSIZE_B + idx_b])
					+ (normcdf(mnow - GGQ_MMEAN, GGQ_MSTD) - normcdf(Vcandidate - GGQ_MMEAN, GGQ_MSTD))*
					(DEBT_M + (1 - DEBT_M)*DEBT_Z + (1 - DEBT_M)*((1 - PROB_LH_ND)*d_qL_ND[idx_y*GRIDSIZE_B + ichoice_prev] + PROB_LH_ND*d_qH_ND[idx_y*GRIDSIZE_B + ichoice_prev]));
                bContinue = false;
				mdeftresh = Vcandidate;
			}

		} // while loop

		CENDupdate = CENDupdate/GGQ_MMASS;
		qHupdate = max(0.0,qHupdate)/GGQ_MMASS;
		def_prob_update = (normcdf(mdeftresh - GGQ_MMEAN, GGQ_MSTD) - normcdf(GGQ_MLB - GGQ_MMEAN, GGQ_MSTD))/GGQ_MMASS;
		qLupdate = max(0.0,qLupdate - HOLDING_COST)/GGQ_MMASS;

	}

}

__device__ REAL_TYPE bisect_zero(REAL_TYPE a, REAL_TYPE b, REAL_TYPE c1, REAL_TYPE c2, REAL_TYPE W1_minus_W2)
{
	REAL_TYPE xlb = a, xub = b, xmid, fmid;
	REAL_TYPE flb = m_root_fun(xlb, c1, c2, W1_minus_W2);
	int i;
	for (i = 0; i < MAXITERS_BISECT; i++)
	{
		xmid = 0.5*(xlb + xub);
		fmid = m_root_fun(xmid, c1, c2, W1_minus_W2);
		if (signbit(flb) == signbit(fmid))
		{
			// same sign
			xlb = xmid;
		}
		else
		{
			// different signs
			xub = xmid;
		}
		if (xub - xlb <= ERRTOL_BISECT)
		{
			break;
		}
	}
	if (i >= MAXITERS_BISECT)
	{
		printf("WARNING: bisect iterations reached MAXITERS_BISECT.");
	}
	return 0.5*(xlb + xub);
}

__device__ REAL_TYPE m_root_fun(REAL_TYPE m, REAL_TYPE U1, REAL_TYPE U2, REAL_TYPE W1_minus_W2)
{
	return POWERFUN(U1 + m, ONE_MINUS_RRA) - POWERFUN(U2 + m, ONE_MINUS_RRA) + W1_minus_W2;
}

__device__ REAL_TYPE CENDupdatefun(REAL_TYPE m, REAL_TYPE U, REAL_TYPE W)
{
	return normpdf(m - GGQ_MMEAN, GGQ_MSTD)
		*(U_SCALING*POWERFUN(U + m,ONE_MINUS_RRA)+W);
}

__device__ REAL_TYPE gauss_legendre_CENDupdate(REAL_TYPE m1, REAL_TYPE m2, REAL_TYPE U, REAL_TYPE W)
{
	REAL_TYPE alpha = 0.5*(m1 + m2);
	REAL_TYPE beta = 0.5*(m2 - m1);

	// return beta*(
	// 	0.0666713443086881375935688*CENDupdatefun(alpha + beta*-0.9739065285171717200779640, U, W)
	// 	+ 0.1494513491505805931457763*CENDupdatefun(alpha + beta*-0.8650633666889845107320967, U, W)
	// 	+ 0.2190863625159820439955349*CENDupdatefun(alpha + beta*-0.6794095682990244062343274, U, W)
	// 	+ 0.2692667193099963550912269*CENDupdatefun(alpha + beta*-0.4333953941292471907992659, U, W)
	// 	+ 0.2955242247147528701738930*CENDupdatefun(alpha + beta*-0.1488743389816312108848260, U, W)
	// 	+ 0.2955242247147528701738930*CENDupdatefun(alpha + beta*0.1488743389816312108848260, U, W)
	// 	+ 0.2692667193099963550912269*CENDupdatefun(alpha + beta*0.4333953941292471907992659, U, W)
	// 	+ 0.2190863625159820439955349*CENDupdatefun(alpha + beta*0.6794095682990244062343274, U, W)
	// 	+ 0.1494513491505805931457763*CENDupdatefun(alpha + beta*0.8650633666889845107320967, U, W)
	// 	+ 0.0666713443086881375935688*CENDupdatefun(alpha + beta*0.9739065285171717200779640, U, W)
    // );
    
    // 21 points
	return 	beta*(0.1460811336496904*CENDupdatefun(alpha + beta*0.0000000000000000, U, W)
		+	0.1445244039899700*CENDupdatefun(alpha + beta*-0.1455618541608951, U, W)
		+	0.1445244039899700*CENDupdatefun(alpha + beta*	0.1455618541608951, U, W)
		+	0.1398873947910731*CENDupdatefun(alpha + beta* -0.2880213168024011, U, W)
		+	0.1398873947910731*CENDupdatefun(alpha + beta*	0.2880213168024011, U, W)
		+	0.1322689386333375*CENDupdatefun(alpha + beta* -0.4243421202074388, U, W)
		+	0.1322689386333375*CENDupdatefun(alpha + beta*	0.4243421202074388, U, W)
		+	0.1218314160537285*CENDupdatefun(alpha + beta* -0.5516188358872198, U, W)
		+	0.1218314160537285*CENDupdatefun(alpha + beta*	0.5516188358872198, U, W)
		+	0.1087972991671484*CENDupdatefun(alpha + beta* -0.6671388041974123, U, W)
		+	0.1087972991671484*CENDupdatefun(alpha + beta*	0.6671388041974123, U, W)
		+	0.0934444234560339*CENDupdatefun(alpha + beta* -0.7684399634756779, U, W)
		+	0.0934444234560339*CENDupdatefun(alpha + beta*	0.7684399634756779, U, W)
		+	0.0761001136283793*CENDupdatefun(alpha + beta* -0.8533633645833173, U, W)
		+	0.0761001136283793*CENDupdatefun(alpha + beta*	0.8533633645833173, U, W)
		+	0.0571344254268572*CENDupdatefun(alpha + beta* -0.9200993341504008, U, W)
		+	0.0571344254268572*CENDupdatefun(alpha + beta*	0.9200993341504008, U, W)
		+	0.0369537897708525*CENDupdatefun(alpha + beta* -0.9672268385663063, U, W)
		+	0.0369537897708525*CENDupdatefun(alpha + beta*	0.9672268385663063, U, W)
		+	0.0160172282577743*CENDupdatefun(alpha + beta* -0.9937521706203895, U, W)
        +	0.0160172282577743*CENDupdatefun(alpha + beta*	0.9937521706203895, U, W));

}
