#ifndef LIQUIDITY_DEFS_H
#define LIQUIDITY_DEFS_H

#include "realtype.h"

#include <string>

#ifndef MAX_GRIDSIZE_Y
#define MAX_GRIDSIZE_Y 256
#endif

#ifndef MAX_GRIDSIZE_B
#define MAX_GRIDSIZE_B 1024
#endif

REAL_TYPE CalcAutarkyOutput(REAL_TYPE yND, REAL_TYPE bdef, REAL_TYPE d_y, REAL_TYPE d_yy, REAL_TYPE d_b, REAL_TYPE d_bb, REAL_TYPE d_yb);
bool GetFieldFromFile(std::string &out_str, char *filename, std::string fieldname_in, char delimiter);
void trimstring(std::string &str, std::string trim_delimiters = " \f\n\r\t\v");
void delimStrToVec(int *z, int &N, std::string s, char delim);
void delimStrToVec(REAL_TYPE *z, int &N, std::string s, char delim);

__global__ void printDeviceConstants();

class parms_bsl_mod
{
public:
	// preferences
	REAL_TYPE rra, beta;

	// numerical parameters
	REAL_TYPE clb, ggq_mDval, ggq_mlb, ggq_mub, ggq_mmean, ggq_mstd, ggq_mmass, errtol_bisect;
	int maxiters_bisect;

	// liquidity discounts
	REAL_TYPE rH, rL, holding_cost;

	// liquidity transitions
	REAL_TYPE prob_liqshock;
	REAL_TYPE prob_meet_mkt_maker_ND, prob_meet_mkt_maker_D;

	// bargaining powers
	REAL_TYPE bargain_power_mkt_maker_ND, bargain_power_mkt_maker_D;

	// debt market
	REAL_TYPE reenterprob, debt_z, debt_m;
	REAL_TYPE debt_writedown_reentry;
	REAL_TYPE recovery_bmax;
	REAL_TYPE max_def_prob;

	// grid dimensions
	REAL_TYPE tfp_lb, tfp_ub;
	REAL_TYPE conv_chk_tfp_lb, conv_chk_tfp_ub;
	REAL_TYPE conv_chk_b_ub;
	int gridsize_tfp, gridsize_b;
	REAL_TYPE bmax;

	// output
	REAL_TYPE rho_tfp, sigma_tfp;
	REAL_TYPE d_y, d_yy, d_b, d_bb, d_yb; // default costs

	// grids
	REAL_TYPE *grid_y_nd, *grid_y_d, *grid_b;
	REAL_TYPE *prob_y;
	REAL_TYPE *grid_uD_yb; 

	// convergence
	int maxiters;
	REAL_TYPE errtolV, errtolQ;
	int NumUpdateRegimes; // e.g. 3
	int *UpdateRegimeIter; // e.g. 100/500
	REAL_TYPE *UpdateWeightOldV, *UpdateWeightOldQ, *UpdateWeightOldDefProb; // 0/0.5/0.9

	int checkparms();

	int readparms(char *filename);

	void initDeviceConstants();

	void displayparms();

	// int read_parms_from_file(const char* filename);

	void clean();

};

// GPU device constants

// preferences
extern __constant__ REAL_TYPE RRA;
extern __constant__ REAL_TYPE BETA;
extern __constant__ REAL_TYPE ONE_MINUS_RRA; // 1-RRA
extern __constant__ REAL_TYPE U_SCALING; // 1/(1-RRA)

// numerical parameters
extern __constant__ REAL_TYPE C_LB;
extern __constant__ REAL_TYPE GGQ_MLB;
extern __constant__ REAL_TYPE GGQ_MUB;
extern __constant__ REAL_TYPE GGQ_MMEAN;
extern __constant__ REAL_TYPE GGQ_MSTD;
extern __constant__ REAL_TYPE GGQ_MMASS;
extern __constant__ REAL_TYPE GGQ_MDEF;
extern __constant__ REAL_TYPE ERRTOL_BISECT;
extern __constant__ REAL_TYPE MAX_DEF_PROB;
extern __constant__ int MAXITERS_BISECT;
extern __constant__ int CONV_CHK_TFP_LB_IDX;
extern __constant__ int CONV_CHK_TFP_UB_IDX;
extern __constant__ int CONV_CHK_B_UB_IDX;

// liquidity discounts
extern __constant__ REAL_TYPE RH;
extern __constant__ REAL_TYPE RL;
extern __constant__ REAL_TYPE HOLDING_COST;

// effective transition types 
//P(H->L) = prob_liqshock
//P(L->H,ND) = prob_meet_mkt_maker*(1-bargain_power_mkt_maker_ND)
//P(L->H,D) = prob_meet_mkt_maker*(1-bargain_power_mkt_maker_D)
extern __constant__ REAL_TYPE PROB_HL;
extern __constant__ REAL_TYPE PROB_LH_ND;
extern __constant__ REAL_TYPE PROB_LH_D;

// debt market
extern __constant__ REAL_TYPE REENTERPROB;
extern __constant__ REAL_TYPE DEBT_Z;
extern __constant__ REAL_TYPE DEBT_M;

// grid dimensions
extern __constant__ int GRIDSIZE_Y;
extern __constant__ int GRIDSIZE_B;

// grids
extern __constant__ REAL_TYPE GRID_Y_ND[MAX_GRIDSIZE_Y];
extern __constant__ REAL_TYPE GRID_B[MAX_GRIDSIZE_B];

// interpolation
extern __constant__ int GRID_B_REENTER_IDX[MAX_GRIDSIZE_B];
extern __constant__ REAL_TYPE GRID_RECOVERY_FRAC[MAX_GRIDSIZE_B];

#endif