#include "liquidity_defs.h"
#include "realtype.h"
#include <fstream>
#include <cstring>
#include <sstream> // convert from integer to string
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <sstream>

#ifdef _WIN32
#define OS_WINDOWS
#elif _WIN64
#define OS_WINDOWS
#elif __linux__
#define OS_NOT_WINDOWS
#elif __unix__
#define OS_NOT_WINDOWS
#else
#error "Unknown compiler in liquidity_io.cpp"
#endif

#ifdef OS_WINDOWS
#include <Windows.h>
#endif

#ifdef OS_NOT_WINDOWS
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif

std::string CreateNewFolder(const char *save_folder_name)
{
	std::string folder_name;
	folder_name.clear();
	std::string root_folder_name;
	root_folder_name.assign(save_folder_name);
	folder_name.assign(save_folder_name);
	std::ostringstream converter;

	bool bContinue = true;
	int count = 0;

	while (bContinue)
	{
#ifdef OS_WINDOWS
		if (CreateDirectory(folder_name.c_str(), NULL) == 0) // 0 if error
#endif
#ifdef OS_NOT_WINDOWS
			if (mkdir(folder_name.c_str(), 0777) != 0)
#endif
			{
				converter.str("");
				count += 1;
				converter << count;
				folder_name = root_folder_name + converter.str();
			}
			else
			{
				bContinue = false;
			}
	}

	printf("Results saved to folder %s\n", folder_name.c_str());

	return folder_name;
}

void write_parms_to_file(parms_bsl_mod &p, const char* filename, int iters, 
	REAL_TYPE err_CVD, REAL_TYPE err_CVND, REAL_TYPE err_qH_ND, REAL_TYPE err_qL_ND, REAL_TYPE err_qH_D, REAL_TYPE err_qL_D)
// NOTE: ORDERING IS IMPORTANT
{

	std::ofstream outfile;

	outfile.open(filename, std::ios::binary | std::ios::out);

	// preferences
	outfile.write((char *)&p.rra, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.beta, 1 * sizeof(REAL_TYPE));

	// numerical parameters
	outfile.write((char *)&p.clb, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mDval, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mlb, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mub, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mmean, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mstd, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.ggq_mmass, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.errtol_bisect, 1 * sizeof(REAL_TYPE));	
	outfile.write((char *)&p.maxiters_bisect, 1 * sizeof(int));

	// liquidity discounts
	outfile.write((char *)&p.rH, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.rL, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.holding_cost, 1 * sizeof(REAL_TYPE));

	// liquidity transitions
	outfile.write((char *)&p.prob_liqshock, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.prob_meet_mkt_maker_ND, 1 * sizeof(REAL_TYPE));
	outfile.write((char*)&p.prob_meet_mkt_maker_D, 1 * sizeof(REAL_TYPE));

	outfile.write((char *)&p.bargain_power_mkt_maker_ND, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.bargain_power_mkt_maker_D, 1 * sizeof(REAL_TYPE));

	// debt market
	outfile.write((char *)&p.reenterprob, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.debt_z, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.debt_m, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.debt_writedown_reentry, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.max_def_prob, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.recovery_bmax, 1 * sizeof(REAL_TYPE));

	// grid dimensions
	outfile.write((char *)&p.gridsize_tfp, 1 * sizeof(int));
	outfile.write((char *)&p.gridsize_b, 1 * sizeof(int));
	outfile.write((char *)&p.bmax, 1 * sizeof(REAL_TYPE));

	// output
	outfile.write((char *)&p.rho_tfp, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.sigma_tfp, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.d_y, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.d_yy, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.d_b, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.d_bb, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.d_yb, 1 * sizeof(REAL_TYPE));

	// grids
	outfile.write((char *)p.grid_y_nd, p.gridsize_tfp * sizeof(REAL_TYPE));
	outfile.write((char *)p.grid_y_d, p.gridsize_tfp*p.gridsize_b * sizeof(REAL_TYPE));
	outfile.write((char *)p.grid_b, p.gridsize_b * sizeof(REAL_TYPE));
	outfile.write((char *)p.prob_y, p.gridsize_tfp * p.gridsize_tfp * sizeof(REAL_TYPE));
	outfile.write((char *)p.grid_uD_yb, p.gridsize_tfp*p.gridsize_b * sizeof(REAL_TYPE));

	// convergence
	outfile.write((char *)&p.maxiters, 1 * sizeof(int));
	outfile.write((char *)&p.errtolV, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.errtolQ, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&p.NumUpdateRegimes, 1 * sizeof(int));
	outfile.write((char *)p.UpdateRegimeIter, (p.NumUpdateRegimes - 1) * sizeof(int));
	outfile.write((char *)p.UpdateWeightOldV, p.NumUpdateRegimes * sizeof(REAL_TYPE));
	outfile.write((char *)p.UpdateWeightOldQ, p.NumUpdateRegimes * sizeof(REAL_TYPE));
	outfile.write((char *)p.UpdateWeightOldDefProb, p.NumUpdateRegimes * sizeof(REAL_TYPE));

	// final convergence indicators
	outfile.write((char *)&iters, 1 * sizeof(int));
	outfile.write((char *)&err_CVD, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&err_CVND, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&err_qH_ND, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&err_qL_ND, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&err_qH_D, 1 * sizeof(REAL_TYPE));
	outfile.write((char *)&err_qL_D, 1 * sizeof(REAL_TYPE));

	outfile.close();

}

template <typename T> void write_2d_to_file(int dim1, int dim2, T *h_x, const char *filename)
// x(a,b)
// output format:
// 1. Na
// 2. Nb
// 4. x
{
	std::ofstream outfile;

	outfile.open(filename, std::ios::binary | std::ios::out);

	// dimensions
	outfile.write((char *)&dim1, 1 * sizeof(int));
	outfile.write((char *)&dim2, 1 * sizeof(int));

	// write object
	outfile.write((char *)h_x, dim1*dim2 * sizeof(T));

	outfile.close();

}

template <typename T> void read_2d_from_file(int &dim1, int &dim2, T *h_x, const char *filename)
// x(a,b,c)
// output format:
// 1. Na
// 2. Nb
// 3. Nc
// 4. x
{
	std::ifstream infile;

	infile.open(filename, std::ios::binary | std::ios::in);

	// dimensions
	infile.read((char *)&dim1, 1 * sizeof(int));
	infile.read((char *)&dim2, 1 * sizeof(int));

	// write object
	infile.read((char *)h_x, dim1*dim2 * sizeof(T));

	infile.close();

}

std::string save_outputs(const char *save_folder_name, parms_bsl_mod &p, REAL_TYPE *CVDs, REAL_TYPE *CVNDs, REAL_TYPE *qH_NDs,
	REAL_TYPE *qL_NDs, REAL_TYPE *qH_Ds, REAL_TYPE *qL_Ds, int iters, 
	REAL_TYPE err_CVD, REAL_TYPE err_CVND, REAL_TYPE err_qH_ND, REAL_TYPE err_qL_ND, REAL_TYPE err_qH_D, REAL_TYPE err_qL_D)
{

	std::string folder_name = CreateNewFolder(save_folder_name);

	std::string parms_file_name = folder_name + "/" + "parms";
	write_parms_to_file(p, parms_file_name.c_str(), iters, err_CVD, err_CVND, err_qH_ND, err_qL_ND, err_qH_D, err_qL_D);

	std::string CVD_file_name = folder_name + "/" + "CVD";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, CVDs, CVD_file_name.c_str()); // y,b

	std::string CVND_file_name = folder_name + "/" + "CVND";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, CVNDs, CVND_file_name.c_str()); // y,b'

	std::string qHND_file_name = folder_name + "/" + "qHND";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qH_NDs, qHND_file_name.c_str()); // y,b'

	std::string qLND_file_name = folder_name + "/" + "qLND";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qL_NDs, qLND_file_name.c_str()); // y,b'

	std::string qHD_file_name = folder_name + "/" + "qHD";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qH_Ds, qHD_file_name.c_str()); // y,b

	std::string qLD_file_name = folder_name + "/" + "qLD";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qL_Ds, qLD_file_name.c_str()); // y,b

	return folder_name;

}

std::string save_outputs_detailed(const char *save_folder_name, parms_bsl_mod &p, REAL_TYPE *CVDs, REAL_TYPE *CVNDs, REAL_TYPE *qH_NDs,
	REAL_TYPE *qL_NDs, REAL_TYPE *qH_Ds, REAL_TYPE *qL_Ds, int iters, 
	REAL_TYPE err_CVD, REAL_TYPE err_CVND, REAL_TYPE err_qH_ND, REAL_TYPE err_qL_ND, REAL_TYPE err_qH_D, REAL_TYPE err_qL_D,
	REAL_TYPE *CVDs_old, REAL_TYPE *CVNDs_old, REAL_TYPE *qH_NDs_old,
	REAL_TYPE *qL_NDs_old, REAL_TYPE *qH_Ds_old, REAL_TYPE *qL_Ds_old, REAL_TYPE *defprob, REAL_TYPE *defprob_old, REAL_TYPE *defthresh)
{
	
	std::string folder_name = save_outputs(save_folder_name, p, CVDs, CVNDs, qH_NDs,
		qL_NDs,qH_Ds, qL_Ds, iters, err_CVD, err_CVND, err_qH_ND, err_qL_ND, err_qH_D, err_qL_D);

	std::string CVD_file_name = folder_name + "/" + "CVD_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, CVDs_old, CVD_file_name.c_str()); // y,b

	std::string CVND_file_name = folder_name + "/" + "CVND_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, CVNDs_old, CVND_file_name.c_str()); // y,b'

	std::string qHND_file_name = folder_name + "/" + "qHND_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qH_NDs_old, qHND_file_name.c_str()); // y,b'

	std::string qLND_file_name = folder_name + "/" + "qLND_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qL_NDs_old, qLND_file_name.c_str()); // y,b'

	std::string qHD_file_name = folder_name + "/" + "qHD_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qH_Ds_old, qHD_file_name.c_str()); // y,b

	std::string qLD_file_name = folder_name + "/" + "qLD_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, qL_Ds_old, qLD_file_name.c_str()); // y,b

	std::string defprob_file_name = folder_name + "/" + "defprob";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, defprob, defprob_file_name.c_str()); // y,b

	std::string defprob_old_file_name = folder_name + "/" + "defprob_old";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, defprob_old, defprob_old_file_name.c_str()); // y,b

	std::string defthresh_file_name = folder_name + "/" + "defthresh";
	write_2d_to_file(p.gridsize_tfp, p.gridsize_b, defthresh, defthresh_file_name.c_str()); // y,b

	return folder_name;

}

//void read_inputs(const char *input_folder_name, REAL_TYPE *CVDs, REAL_TYPE *CVNDs, REAL_TYPE *qH_NDs,
//	REAL_TYPE *qL_NDs, REAL_TYPE *qH_Ds, REAL_TYPE *qL_Ds)
//{
//
//	int dim1, dim2;
//
//	std::string folder_name;
//	folder_name.assign(input_folder_name);
//
//	std::string CVD_file_name = folder_name + "/" + "CVD";
//	read_2d_from_file(dim1, dim2, CVDs, CVD_file_name.c_str()); // y,b
//
//	std::string CVND_file_name = folder_name + "/" + "CVND";
//	read_2d_from_file(dim1, dim2, CVNDs, CVND_file_name.c_str()); // y,b'
//
//	std::string qHND_file_name = folder_name + "/" + "qHND";
//	read_2d_from_file(dim1, dim2, qH_NDs, qHND_file_name.c_str()); // y,b'
//
//	std::string qLND_file_name = folder_name + "/" + "qLND";
//	read_2d_from_file(dim1, dim2, qL_NDs, qLND_file_name.c_str()); // y,b'
//
//	std::string qHD_file_name = folder_name + "/" + "qHD";
//	read_2d_from_file(dim1, dim2, qH_Ds, qHD_file_name.c_str()); // y,b
//
//	std::string qLD_file_name = folder_name + "/" + "qLD";
//	read_2d_from_file(dim1, dim2, qL_Ds, qLD_file_name.c_str()); // y,b
//
//}