function [model,p] = ProcessMexGPUOutputs_liq(p, model_mex)
% it's assumed that model has already been converted from c++ storage
% format to matlab storage format using 
% conv_outputs = ConvertCPPtoMAT(p,model_outputs)

    %% load data
    
    p.grid_uD_yb = max(p.clb, model_mex.grid_y_d).^(1 - p.rra)/(1-p.rra);
    p.grid_y_nd = model_mex.grid_y_nd;
    p.grid_y_d = model_mex.grid_y_d;
    p.ggq_mmass = normcdf(p.ggq_mub,p.ggq_mmean,p.ggq_mstd) - normcdf(p.ggq_mlb,p.ggq_mmean,p.ggq_mstd);
%     p.prob_y = model_mex.prob_y;
%     p.prob_mpr = model_mex.prob_mpr;
%     p.prob_RN_y_mpr_ynext = model_mex.prob_RN_y_mpr_ynext;

    % transition probs for (y,mpr)
    sparse_i = [];
    sparse_j = [];
    sparse_v = [];
    sparse_v_RN = [];
    
    for idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
        [idx_y, idx_mpr] = ind2sub([p.gridsize_tfp p.gridsize_mpr], idx_y_mpr);
        idx_y_next = find(model_mex.prob_y(idx_y,:)>0);
        idx_mpr_next = find(model_mex.prob_mpr(idx_mpr,:)>0);
        [IDX_Y,IDX_MPR] = ndgrid(idx_y_next(:),idx_mpr_next(:));
        IDX_Y_MPR = sub2ind([p.gridsize_tfp p.gridsize_mpr],IDX_Y(:),IDX_MPR(:));
        [P_Y,P_MPR] = ndgrid(model_mex.prob_y(idx_y,idx_y_next)', model_mex.prob_mpr(idx_mpr,idx_mpr_next)');
        [P_Y_RN,P_MPR_RN] = ndgrid(squeeze(model_mex.prob_RN_y_mpr_ynext(idx_y,idx_mpr,idx_y_next)), model_mex.prob_mpr(idx_mpr,idx_mpr_next)');
        sparse_i = [sparse_i; idx_y_mpr*ones(numel(IDX_Y_MPR),1)];
        sparse_j = [sparse_j; IDX_Y_MPR];
        sparse_v = [sparse_v; P_Y(:).*P_MPR(:)];
        sparse_v_RN = [sparse_v_RN; P_Y_RN(:).*P_MPR_RN(:)];
    end
    
    p.prob_y = model_mex.prob_y;
    p.prob_mpr = model_mex.prob_mpr;
    p.prob_y_mpr = sparse(sparse_i,sparse_j,sparse_v,p.gridsize_tfp*p.gridsize_mpr,p.gridsize_tfp*p.gridsize_mpr);
    p.prob_RN_y_mpr = sparse(sparse_i,sparse_j,sparse_v_RN,p.gridsize_tfp*p.gridsize_mpr,p.gridsize_tfp*p.gridsize_mpr);
    
    % input data ordered as (y,mpr,b)
    CVD = model_mex.CVD;
    CVND = model_mex.CVND;
    qHD = model_mex.qDH;
    qLD = model_mex.qDL;
    qHND = model_mex.qNDH;
    qLND = model_mex.qNDL;
    defprob = model_mex.defprob;
    defthresh = model_mex.defthresh;
    
    CVD_old = reshape(permute(model_mex.CVD,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    CVND_old = reshape(permute(model_mex.CVND,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qHD_old = reshape(permute(model_mex.qDH,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qLD_old = reshape(permute(model_mex.qDL,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qHND_old = reshape(permute(model_mex.qNDH,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qLND_old = reshape(permute(model_mex.qNDL,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    defprob_old = reshape(permute(model_mex.defprob,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    defthresh_old = reshape(permute(model_mex.defthresh,[3 1 2]),p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);

    %% solve model
    
    VD_mat = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    debt_policy_mat = cell(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    default_threshold_eps_mat = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    ceND_new_integrate_eps_mat = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qND_H_new_integrate_eps_mat = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qND_L_new_integrate_eps_mat = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    defprob_new_integrate_eps = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qND_diff = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    qD_diff = zeros(p.gridsize_b,p.gridsize_tfp*p.gridsize_mpr);
    
    parfor idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
        [VD_mat(:,idx_y_mpr),debt_policy_mat(:,idx_y_mpr),default_threshold_eps_mat(:,idx_y_mpr),ceND_new_integrate_eps_mat(:,idx_y_mpr),...
            qND_H_new_integrate_eps_mat(:,idx_y_mpr),qND_L_new_integrate_eps_mat(:,idx_y_mpr),defprob_new_integrate_eps(:,idx_y_mpr),...
            qND_diff(:,idx_y_mpr),qD_diff(:,idx_y_mpr)] = ...
                VFI_given_y_mpr(p,idx_y_mpr,CVND_old(:,idx_y_mpr),CVD_old(:,idx_y_mpr),...
                qHND_old(:,idx_y_mpr),qLND_old(:,idx_y_mpr),qHD_old(:,idx_y_mpr),qLD_old(:,idx_y_mpr),defprob_old(:,idx_y_mpr))
    end
    
    VD_mat = permute(VD_mat,[2 1]);
    debt_policy_mat = permute(debt_policy_mat,[2 1]);
    default_threshold_eps_mat = permute(default_threshold_eps_mat,[2 1]);
    ceND_new_integrate_eps_mat = permute(ceND_new_integrate_eps_mat,[2 1]);
    qND_H_new_integrate_eps_mat = permute(qND_H_new_integrate_eps_mat,[2 1]);
    qND_L_new_integrate_eps_mat = permute(qND_L_new_integrate_eps_mat,[2 1]);
    defprob_new_integrate_eps = permute(defprob_new_integrate_eps,[2 1]);
    qND_diff = permute(qND_diff,[2 1]);
    qD_diff = permute(qD_diff,[2 1]);
    
    qHD_old = permute(qHD_old,[2 1]);
    qLD_old = permute(qLD_old,[2 1]);
    
    [ceD_mat,ceC_mat,qND_H_mat,qND_L_mat,qD_H_mat,qD_L_mat,defprob_mat,ellND,ellD] = UpdateVals(p,VD_mat,ceND_new_integrate_eps_mat,...
        qND_H_new_integrate_eps_mat,qND_L_new_integrate_eps_mat,defprob_new_integrate_eps,qHD_old,qLD_old,qND_diff,qD_diff);
    
    %% reshape for output
    
    VD_mat = reshape(VD_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    ceD_mat = reshape(ceD_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    ceC_mat = reshape(ceC_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    qND_H_mat = reshape(qND_H_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    qND_L_mat = reshape(qND_L_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    qD_H_mat = reshape(qD_H_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    qD_L_mat = reshape(qD_L_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    defprob_mat = reshape(defprob_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    ellND = reshape(ellND,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    ellD = reshape(ellD,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    default_threshold_eps_mat = reshape(default_threshold_eps_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    debt_policy_mat = reshape(debt_policy_mat,[p.gridsize_tfp p.gridsize_mpr p.gridsize_b]);
    
    %% clean up

    err_CVD = max(reshape(abs(ceD_mat - CVD),[],1));
    err_CVND = max(reshape(abs(ceC_mat - CVND),[],1));
    err_qNDH = max(reshape(abs(qHND - qND_H_mat),[],1));
    err_qNDL = max(reshape(abs(qLND - qND_L_mat),[],1));
    err_qDH = max(reshape(abs(qHD - qD_H_mat),[],1));
    err_qDL = max(reshape(abs(qLD - qD_L_mat),[],1));
    err_defprob = max(reshape(abs(defprob - defprob_mat),[],1));
    err_defthresh = max(reshape(abs(defthresh - default_threshold_eps_mat),[],1));
    
    
    %% save data
    model.VD = VD_mat;
    model.CVD = ceD_mat;
    model.CVND = ceC_mat;
    model.policy = debt_policy_mat;
    model.defthresh = default_threshold_eps_mat;
    model.defprob = defprob_mat;
    model.qNDH = qND_H_mat;
    model.qNDL = qND_L_mat;
    model.qDH = qD_H_mat;
    model.qDL = qD_L_mat;
    model.ellND = ellND;
    model.ellD = ellD;
    
    model.err_CVD = err_CVD;
    model.err_CVND = err_CVND;
    model.err_qNDH = err_qNDH;
    model.err_qNDL = err_qNDL;
    model.err_qDH = err_qDH;
    model.err_qDL = err_qDL;
    model.err_defprob = err_defprob;
    model.err_defthresh = err_defthresh;
    
    %% compute additional output
    
    model = ComputeModelAuxiliaryEfficient(p,model);

end

function [VD_new,debt_policy,default_threshold_eps,ceND_new_integrate_eps,...
    qND_H_new_integrate_eps,qND_L_new_integrate_eps,defprob_new_integrate_eps,qND_diff,qD_diff] = ...
        VFI_given_y_mpr(p,idx_y_mpr,ceC,ceD,qND_H,qND_L,qD_H,qD_L,defprob)

    [idx_y,idx_mpr] = ind2sub([p.gridsize_tfp p.gridsize_mpr],idx_y_mpr);
    ceC = reshape(ceC,p.gridsize_b,1);
    ceD = reshape(ceD,p.gridsize_b,1);
    qND_H = reshape(qND_H,p.gridsize_b,1);
    qND_L = reshape(qND_L,p.gridsize_b,1);
    qD_H = reshape(qD_H,p.gridsize_b,1);
    qD_L = reshape(qD_L,p.gridsize_b,1);
    defprob = reshape(defprob,p.gridsize_b,1);

    VD_new = zeros(p.gridsize_b,1);
    debt_policy = cell(p.gridsize_b,1);
    default_threshold_eps = zeros(p.gridsize_b,1);
    ceND_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_H_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_L_new_integrate_eps = zeros(p.gridsize_b,1);
    defprob_new_integrate_eps = zeros(p.gridsize_b,1);
    qND_diff = zeros(p.gridsize_b,1);
    qD_diff = zeros(p.gridsize_b,1);

    for idx_b = 1:p.gridsize_b
        [VD_new(idx_b),debt_policy{idx_b},default_threshold_eps(idx_b),ceND_new_integrate_eps(idx_b),...
            qND_H_new_integrate_eps(idx_b),qND_L_new_integrate_eps(idx_b),defprob_new_integrate_eps(idx_b),...
                qND_diff(idx_b),qD_diff(idx_b)] = ...
                VFI_given_y_b(p,idx_y,idx_b,ceC,ceD,qND_H,qND_L,qD_H,qD_L,defprob);
    end

end

function [ceDwithhaircut,ceC,qND_H,qND_L,qD_H,qD_L,defprob,ellND,ellD] = UpdateVals(p,VD_new,ceC_new_integrate_eps,...
    qND_H_new_integrate_eps,qND_L_new_integrate_eps,defprob_new_integrate_eps,qD_H_old,qD_L_old,qND_diff,qD_diff)
%   qND_diff = (1-m)*qdiff*NODEF + qdiff*DEF, integrated over smoothing shock
%   qD_diff = qdiff, q=qD

    % update value functions
    ceD1 = p.prob_y_mpr*VD_new; % E[VD(y',b)|y]
    ceC = p.prob_y_mpr*ceC_new_integrate_eps; %E[max{VD(y',b'),VC(y',e',b')}|y]
    ceC_haircut = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b); % reentering at (1-L)*(1-deltaL)*b
    for idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
        ceC_haircut(idx_y_mpr,:) = interp1(p.grid_b,ceC(idx_y_mpr,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest'); % linear is most robust
    end
    ceDwithhaircut = (1-p.reenterprob)*ceD1 + p.reenterprob*ceC_haircut;
    
    % update bond prices, check these for consistency...
    qND_H = p.prob_RN_y_mpr*qND_H_new_integrate_eps/(1+p.r);
    qND_L = p.prob_RN_y_mpr*qND_L_new_integrate_eps/(1+p.r);
    
    qD_H1 = p.prob_RN_y_mpr*((1-p.prob_liqshock)*qD_H_old+p.prob_liqshock*qD_L_old);
    qD_L1 = p.prob_RN_y_mpr*max(0,(1-p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D))*qD_L_old+p.prob_meet_mkt_maker_D*(1-p.bargain_power_mkt_maker_D)*qD_H_old - p.holding_cost);
    qND_H_haircut = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    qND_L_haircut = zeros(p.gridsize_tfp*p.gridsize_mpr,p.gridsize_b);
    for idx_y_mpr = 1:p.gridsize_tfp*p.gridsize_mpr
        qND_H_haircut(idx_y_mpr,:) = interp1(p.grid_b,qND_H(idx_y_mpr,:),min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b).*p.grid_b,'nearest');
        qND_L_haircut(idx_y_mpr,:) = interp1(p.grid_b,qND_L(idx_y_mpr,:),min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b).*p.grid_b,'nearest');
    end
    
    rec2 = permute(repmat(min(1-p.debt_writedown_reentry,p.recovery_bmax./p.grid_b),[1 p.gridsize_tfp*p.gridsize_mpr]),[2 1]);
    qD_H = max(0,(1-p.reenterprob)*qD_H1/(1+p.r) ...
        + p.reenterprob*rec2.*qND_H_haircut);
    qD_L = max(0,(1-p.reenterprob)*qD_L1/(1+p.r) ...
        + p.reenterprob*rec2.*qND_L_haircut);
    
    defprob = p.prob_y_mpr*defprob_new_integrate_eps;
    
    Eqdiff = p.prob_RN_y_mpr*qND_diff;
    Eqdiff_writedown = zeros(size(Eqdiff));
    for i = 1:p.gridsize_tfp*p.gridsize_mpr
        Eqdiff_writedown(i,:) = interp1(p.grid_b,Eqdiff(i,:),min((1-p.debt_writedown_reentry)*p.grid_b,p.recovery_bmax),'nearest');
    end
    ellND = p.prob_liqshock*Eqdiff./qND_H;
    ellD = p.prob_liqshock*((1-p.reenterprob)*(p.prob_RN_y_mpr*qD_diff) ... 
        + p.reenterprob*rec2.*Eqdiff_writedown)./qD_H;

end