function policy = ggq_topdown(Clb,RRA,mlb,mub,clist,Wlist,ErrTol_cross_pts)
% it is assumed that cs0 has already been sorted in ascending order
% it's assumed that all cs0+p.m_lb are positive.
% problem is max u(c+m)+W where u is concave.
% p contains: Beta, m_lb, m_ub, fsolve_options, C_lb, RRA

    % initiate at mub
    [Vnow,cnow,Wnow,idxmax] = max_lowest_c(RRA,clist,Wlist,mub,Clb);
    idx_orig = 1:numel(clist);
    
    if isnan(Vnow)
        
        % there's no feasible choice
        policy.m_mesh = [mlb,mub];
        policy.choice_index = NaN; % NaN values indicate infeasible choice
        policy.Vs = [NaN,NaN];
        policy.cs = NaN;
        policy.Ws = NaN;
        
    else
        
        % there's at least one feasible choice
        
        policy.m_mesh = mub;
        policy.choice_index = idxmax;
        policy.Vs = Vnow;
        policy.cs = cnow;
        policy.Ws = Wnow;
        
        idx_keep = find(clist>cnow);
        idx_orig = idx_orig(idx_keep);
        clist = clist(idx_keep);
        Wlist = Wlist(idx_keep);
        
        bContinue = ~isempty(idx_keep);
        
        if ~bContinue
            policy = conclude_policy(RRA,policy,mlb,cnow,Wnow,Clb);
        end
        
        
        mnow = mub;
        
        while bContinue
            
            update_info = Find_m_crossing_top_down(Clb,RRA,mlb,mnow,cnow,Wnow,clist,Wlist,ErrTol_cross_pts);
            
            if update_info.bContinueAlg
                
                policy.m_mesh = [update_info.mnext,policy.m_mesh];
                policy.choice_index = [idx_orig(update_info.idxmax),policy.choice_index];
                policy.Vs = [update_info.Vnext,policy.Vs];
                policy.cs = [update_info.cnext,policy.cs];
                policy.Ws = [update_info.Wnext,policy.Ws];
                
                mnow = update_info.mnext;
                cnow = update_info.cnext;
                Wnow = update_info.Wnext;
                idx_keep = find(clist>cnow);
                idx_orig = idx_orig(idx_keep);
                clist = clist(idx_keep);
                Wlist = Wlist(idx_keep);
                
                if isempty(clist)
                    
                    bContinue = false;
                    policy = conclude_policy(RRA,policy,mlb,cnow,Wnow,Clb);
                    
                end
                
            else
                
                bContinue = false;
                policy = conclude_policy(RRA,policy,mlb,cnow,Wnow,Clb);
                
            end
            
        end
        
    end
    
end

function policy = conclude_policy(RRA,policy,mlb,cnow,Wnow,Clb)

    m_feas = Clb - cnow;
    
    if m_feas<=mlb
        policy.m_mesh = [mlb,policy.m_mesh];
        policy.Vs = [f(RRA,cnow+mlb,Wnow),policy.Vs];
    else
        policy.choice_index = [NaN,policy.choice_index];
        policy.cs = [NaN,policy.cs];
        policy.Ws = [NaN,policy.Ws];
        policy.m_mesh = [mlb,m_feas,policy.m_mesh];
        policy.Vs = [NaN,f(RRA,cnow+m_feas,Wnow),policy.Vs];
    end

end

function update_info = ...
    Find_m_crossing_top_down(Clb,RRA,mlb,m_current,c_current,W_current,clist,Wlist,ErrTol_cross_pts)

    if any(clist<=c_current)
        error('potential choices of c should be greater than c_current');
        % NOTE: clist and Wlist do not overlap with c_current and W_current!
    end

    m1 = max(Clb - c_current, mlb);
    V_current_m1 = f(RRA,c_current + m1,W_current);
    
    % all choices in clist will be feasible on m>=m1
    mnext = -Inf;
    cnext = Inf;
    Wnext = NaN;
    Vnext = NaN;
    idx = NaN;
    bCross = false;
    for i = 1:numel(clist)
        if f(RRA,clist(i) + m1,Wlist(i))>V_current_m1
            % find crossing point
            [m_switch,V_switch] = Bisect(RRA,m1,m_current,c_current,W_current,clist(i),Wlist(i),ErrTol_cross_pts);
            if m_switch>mnext
                bCross = true;
                mnext = m_switch;
                cnext = clist(i);
                Wnext = Wlist(i);
                Vnext = V_switch;
                idx = i;
            elseif m_switch==mnext
                % choose one with lowest c
                if clist(i)<cnext
                    bCross = true;
                    cnext = clist(i);
                    Wnext = Wlist(i);
                    Vnext = V_switch;
                    idx = i;
                end
            end
        end
    end
    
    if bCross
        
        update_info.bContinueAlg = true;
        update_info.mnext = mnext;
        update_info.cnext = cnext;
        update_info.Wnext = Wnext;
        update_info.Vnext = Vnext;
        update_info.idxmax = idx;
        
    else
        
        % didn't cross, terminate algorithm unless current choice is non
        % feasible for low m
       
        if m1<=mlb % m1=mlb

            % all choices in clist is feasible
            update_info.bContinueAlg = false;

        else % m1>mlb

            % for m<m1, the current choice will not be feasible
            update_info.bContinueAlg = true;
            update_info.mnext = m1;
            
            % find the best choice at m1 excluding the current choice
            [Vnext,cnext,Wnext,idx] = max_lowest_c(RRA,clist,Wlist,m1);
            
            update_info.cnext = cnext;
            update_info.Wnext = Wnext;
            update_info.Vnext = Vnext;
            update_info.idxmax = idx;

        end
        
    end

end

%% back up bisection algorithm in case the fsolve doesn't work
function [m_switch,V_switch] = Bisect(RRA,mlb0,mub0,c1,betatimesW1,c2,betatimesW2,ErrTol_cross_pts)
    % need to handle both cases 3 and 4

    mlb = mlb0;
    mub = mub0;

    err_mub = f(RRA,c2+mub,betatimesW2) - f(RRA,c1+mub,betatimesW1);

    bContinue = true;

    while bContinue

        m_switch = (mlb+mub)/2;
        err_mswitch = f(RRA,c2+m_switch,betatimesW2) - f(RRA,c1+m_switch,betatimesW1);

        if sign(err_mub)*sign(err_mswitch)<=0
            mlb = m_switch;
            % err_mlb = err_mswitch;
        else
            mub = m_switch;
            err_mub = err_mswitch;
        end

        err = mub-mlb;

        % debug
%             err

        if err<ErrTol_cross_pts
            bContinue = false;
        end

    end

    m_switch = (mlb + mub)/2;
    V_switch = f(RRA,c2+m_switch,betatimesW2);

end

%% general valuation tools
function [V,c,W,idx] = max_lowest_c(RRA,clist,Wlist,m,Clb)

    idx_keep = find(clist + m>=Clb);
    
    if ~isempty(idx_keep)
    
        Vlist = f(RRA,clist(idx_keep) + m,Wlist(idx_keep));
        V = max(Vlist);

        idx_temp = find(V==Vlist);

        if numel(idx_temp)>1

            % choose the one with the lowest c
            [c,idx2] = min(clist(idx_keep(idx_temp)));
            idx = idx_keep(idx_temp(idx2));
            W = Wlist(idx);

        else

            idx = idx_keep(idx_temp);
            c = clist(idx);
            W = Wlist(idx);

        end
    
    else
        
        V = NaN;
        c = NaN;
        W = NaN;
        idx = NaN;
        
    end

end


function V = f(RRA,C,W)
    
    V = C.^(1-RRA)/(1-RRA) + W;

end