#ifndef LIQUIDITY_RISK_AVERSE_MEX_DEFS_H
#define LIQUIDITY_RISK_AVERSE_MEX_DEFS_H

#include "realtype.h"
#include "mex.h"

#ifndef MAX_GRIDSIZE_Y
#define MAX_GRIDSIZE_Y 256
#endif

#ifndef MAX_GRIDSIZE_B
#define MAX_GRIDSIZE_B 512
#endif

#ifndef MAX_GRIDSIZE_X
#define MAX_GRIDSIZE_X 32
#endif

REAL_TYPE CalcAutarkyOutput(REAL_TYPE yND, REAL_TYPE d_y, REAL_TYPE d_yy);

int ReadScalarFromMatlabStruc(double& ReadVal, const mxArray* mxPtr, const char* fieldname);
int ReadVectorFromMatlabStruc(double* ReadVal, const mxArray* mxPtr, int Nread, const char* fieldname);


// __global__ void printDeviceConstants();

// class for the extended model
class parms_extend_mod
{
public:
	// preferences
	REAL_TYPE rra, beta;

	// numerical parameters
	REAL_TYPE clb, ggq_mDval, ggq_mlb, ggq_mub, ggq_mmean, ggq_mstd, ggq_mmass, errtol_bisect;
	int maxiters_bisect;

	// liquidity discounts
	REAL_TYPE r, holding_cost;

	// liquidity transitions
	REAL_TYPE prob_liqshock;
	REAL_TYPE prob_meet_mkt_maker_ND, prob_meet_mkt_maker_D;

	// bargaining powers
	REAL_TYPE bargain_power_mkt_maker_ND, bargain_power_mkt_maker_D;

	// debt market
	REAL_TYPE reenterprob, debt_z, debt_m;
	REAL_TYPE debt_writedown_reentry;
	REAL_TYPE recovery_bmax;
	REAL_TYPE max_def_prob;

	// grid dimensions
	REAL_TYPE width_std_tfp;
	REAL_TYPE width_std_mpr;
	REAL_TYPE conv_chk_width_std_tfp, conv_chk_width_std_mpr, conv_chk_b_ub;
	int gridsize_tfp, gridsize_mpr, gridsize_b;
	REAL_TYPE bmax;

	// output
	REAL_TYPE rho_tfp, sigma_tfp;
	REAL_TYPE d_y, d_yy; // default costs

	// market price of risk
	REAL_TYPE mean_mpr, rho_mpr, sigma_mpr, corr_sdf_tfp_shock;

	// grids
	REAL_TYPE* grid_mpr; // size Nx
	REAL_TYPE* grid_y_nd, * grid_y_d, * grid_uD_y; // size Nz
	REAL_TYPE* grid_b; // size Nb
	REAL_TYPE* P_y; // size Nz*Nz
	REAL_TYPE* P_mpr; // size Nx*Nx
	REAL_TYPE* Q_y_mpr_ynext;  // size Nz*Nx*Nz

	// convergence
	int maxiters;
	REAL_TYPE errtolV, errtolQ;
	int NumUpdateRegimes; // e.g. 3
	int *UpdateRegimeIter; // e.g. 100/500
	REAL_TYPE *UpdateWeightOldV, *UpdateWeightOldQ, *UpdateWeightOldDefProb; // 0/0.5/0.9

	int checkparms();

	int readparms(const mxArray* mxPtr); // defined in liquidity_io_matlab.cpp

	void matlab_display_cpu_parms();

	void initDeviceConstants(); // defined in vfi

	void clean();

};

#endif