#include "mex.h"
#include <cstdio>
#include <cstdlib>
#include "liquidity_risk_averse_mex_defs.h"
#include "matrix.h"

void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]) {

	if (nrhs != 1)
	{
		mexErrMsgIdAndTxt("mexTestReadParms:nrhs", "Should have exactly 1 input");
	}

	if (nlhs != 1)
	{
		mexErrMsgIdAndTxt("mexTestReadParms:nlhs", "Should have exactly 1 output");
	}

	if (!mxIsStruct(prhs[0])) {
		mexErrMsgIdAndTxt("mexTestReadParms:InputNotStruc", "Input must be a structure.");
	}

	parms_extend_mod p;

	if (p.readparms(prhs[0]) == EXIT_SUCCESS)
	{

		p.matlab_display_cpu_parms();

		// run vfi

		// output
		const char* fnames[5] = {"grid_mpr", "grid_y_nd", "P_y", "P_mpr", "Q_y_mpr_ynext"};

		plhs[0] = mxCreateStructMatrix(1, 1, 5, fnames);

		mxArray* mxArray1;
		double* tmp_dbl;

		mxArray1 = mxCreateDoubleMatrix(p.gridsize_mpr, 1, mxREAL);
		tmp_dbl = mxGetPr(mxArray1);
		for (int i = 0; i < p.gridsize_mpr; i++)
		{
			tmp_dbl[i] = (double)p.grid_mpr[i];
		}
		mxSetField(plhs[0], 0, "grid_mpr", mxArray1);

		mxArray1 = mxCreateDoubleMatrix(p.gridsize_tfp, 1, mxREAL);
		tmp_dbl = mxGetPr(mxArray1);
		for (int i = 0; i < p.gridsize_tfp; i++)
		{
			tmp_dbl[i] = (double)p.grid_y_nd[i];
		}
		mxSetField(plhs[0], 0, "grid_y_nd", mxArray1);

		mxArray1 = mxCreateDoubleMatrix(p.gridsize_tfp*p.gridsize_tfp, 1, mxREAL);
		tmp_dbl = mxGetPr(mxArray1);
		for (int i = 0; i < p.gridsize_tfp* p.gridsize_tfp; i++)
		{
			tmp_dbl[i] = (double)p.P_y[i];
		}
		mxSetField(plhs[0], 0, "P_y", mxArray1);

		mxArray1 = mxCreateDoubleMatrix(p.gridsize_mpr * p.gridsize_mpr, 1, mxREAL);
		tmp_dbl = mxGetPr(mxArray1);
		for (int i = 0; i < p.gridsize_mpr * p.gridsize_mpr; i++)
		{
			tmp_dbl[i] = (double)p.P_mpr[i];
		}
		mxSetField(plhs[0], 0, "P_mpr", mxArray1);

		mxArray1 = mxCreateDoubleMatrix(p.gridsize_tfp * p.gridsize_mpr * p.gridsize_tfp, 1, mxREAL);
		tmp_dbl = mxGetPr(mxArray1);
		for (int i = 0; i < p.gridsize_tfp * p.gridsize_mpr * p.gridsize_tfp; i++)
		{
			tmp_dbl[i] = (double)p.Q_y_mpr_ynext[i];
		}
		mxSetField(plhs[0], 0, "Q_y_mpr_ynext", mxArray1);

	}
	else
	{
		mexPrintf("Error reading parameters.\n");
	}

}