***BONDS PANEL***

*----------------------------------------------------------------------------------------
*0. Inputs
*----------------------------------------------------------------------------------------

clear all
set more off
cd "C:\Users\juanp\Dropbox\Proyectos\3.Submitted\PX.Liquidity.sovereign.debt.markets\sovereign.liquidity\Notes\Replication.PX.22"
*cd "F:\Bonds Panel\1993-2004"

*----------------------------------------------------------------------------------------
*1. Loading data, setting panel, and defining variables
*----------------------------------------------------------------------------------------

*1.1. Load Bloomberg data

*This database contains the wide panel from "Bonds Revision Final.xlsx".
*After reshaping to long format, I merge to get the identifiers (step 6).
use "Bond Prices Values.dta", clear 

*1.2. Reshape to long format

rename px_bid var2
rename px_ask var3
rename px_last var4
rename px_open var5
rename px_mid var6
rename px_high var7
rename px_low var8
rename px_official_close var9
rename yld_ytm_mid var10


forvalues x = 2 (9) 290{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_bid`i'
}

forvalues x = 3 (9) 291{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_ask`i'
}

forvalues x = 4 (9) 292{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_last`i'
}

forvalues x = 5 (9) 293{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_open`i'
}

forvalues x = 6 (9) 294{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_mid`i'
}

forvalues x = 7 (9) 295{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_high`i'
}


forvalues x = 8 (9) 296{
cap destring var`x', replace force
local i=int(`x'/9)+1
rename var`x' px_low`i'
}

forvalues x = 9 (9) 297{
cap destring var`x', replace force
local i=int(`x'/9)
rename var`x' px_official_close`i'
}

forvalues x = 10 (9) 298{
cap destring var`x', replace force
local i=int(`x'/9)
rename var`x' yld_ytm_mid`i'
}

reshape long px_bid px_ask px_last px_open px_mid px_high px_low px_official_close yld_ytm_mid, i(dates) j(bond_aux)

gen date=date(dates, "MDY")
format date %td
drop dates

*1.2. Set panel dataset

order bond_aux date
sort bond_aux date
xtset bond_aux date


*-----------------------------------------------------------------------------------------
*2. Merge with bond characteristics
*-----------------------------------------------------------------------------------------
*This file contains the "bond characteristics" module in "Bonds Revision Final.xlsx" (A1:L488)
*We are focusing now on the bonds of the 2020 exchange ONLY.

preserve
tempfile bond_char
import excel "Argentina 1993 to 2004 Global Issued Bonds and Prices Final.xlsx", sheet("Bonds") firstrow clear
gen bond_aux=_n
save `bond_char', replace
restore

sort bond_aux date
merge m:1 bond_aux using `bond_char'
drop _merge

*-----------------------------------------------------------------------------------------
*3. Data cleaning
*-----------------------------------------------------------------------------------------

*3.1. Maturity date
rename Maturity maturity_date


*3.2. Keep only bonds with complete ISIN data
order ISIN date
drop if ISIN=="#N/A Field Not Applicable"
drop if ISIN==""
sort ISIN date
drop bond_aux

encode ISIN, gen(idpanel)
order idpanel date
xtset idpanel date
sort idpanel date


*-----------------------------------------------------------------------------------------
*4. Proportional bid-ask spread
*-----------------------------------------------------------------------------------------

gen ba_spread=((px_ask-px_bid)/((px_ask+px_bid)/2))*100*100

drop if ba_spread<0

save panel_bonds_19932004, replace
